@@ -39,7 +39,9 @@ admin.site.register(
         models.SearchComponent,
         models.IIIFManifest,
         models.GroupMapSettings,
+        models.NodeGroup,
+        models.SpatialView,
     ]
 )
 
-admin.site.register([models.Plugin, models.NodeGroup], GuardedAdmin)
+admin.site.register([models.Plugin, models.ETLModule], GuardedAdmin)
