@@ -36,6 +36,7 @@ define([
             viewModel.ontologyClasses = ko.observable(data['ontologyClasses']);
             viewModel.cardComponents = data.cardComponents;
             viewModel.appliedFunctions = ko.observable(data['appliedFunctions']);
+            viewModel.primaryDescriptorFunction = ko.observable(data['primaryDescriptorFunction']);
 
             var resources = ko.utils.arrayFilter(viewData.graphs, function(graph) {
                 return graph.isresource;
@@ -211,6 +212,7 @@ define([
             viewModel.cardTree = new CardTreeViewModel({
                 graph: viewModel.graph,
                 appliedFunctions: viewModel.appliedFunctions,
+                primaryDescriptorFunction: viewModel.primaryDescriptorFunction,
                 graphModel: viewModel.graphModel
             });
 
@@ -218,6 +220,7 @@ define([
                 graph: viewModel.graph,
                 graphModel: viewModel.graphModel,
                 appliedFunctions: viewModel.appliedFunctions,
+                primaryDescriptorFunction: viewModel.primaryDescriptorFunction,
                 multiselect: true
             });
 
@@ -251,6 +254,7 @@ define([
                 loading: viewModel.loading,
                 node: viewModel.selectedNode,
                 appliedFunctions: viewModel.appliedFunctions,
+                primaryDescriptorFunction: viewModel.primaryDescriptorFunction,
                 restrictedNodegroups: data.restrictedNodegroups
             });
 
@@ -281,7 +285,11 @@ define([
                 onReset: function() {
                     var graph = koMapping.toJS(viewModel.graphSettingsViewModel.graph);
                     viewModel.report.resetConfigs(graph.config);
-                    viewModel.report.get('template_id')(graph["template_id"]);
+
+                    // only reset the template if it's been changed
+                    if (viewModel.report.get('template_id')() !== graph["template_id"]) {
+                        viewModel.report.get('template_id')(graph["template_id"]);
+                    }
                 }
             });
 
@@ -315,6 +323,7 @@ define([
                 cardTree: viewModel.cardTree,
                 permissionTree: viewModel.permissionTree,
                 appliedFunctions: viewModel.appliedFunctions,
+                primaryDescriptorFunction: viewModel.primaryDescriptorFunction,
                 restrictedNodegroups: data.restrictedNodegroups
             });
 
