@@ -23,7 +23,7 @@ import uuid
 from copy import copy, deepcopy
 from django.core.cache import cache
 from django.core.exceptions import ObjectDoesNotExist
-from django.db import transaction
+from django.db import transaction, connection
 from django.db.utils import IntegrityError
 from arches.app.models import models
 from arches.app.models.resource import Resource
@@ -188,6 +188,7 @@ class Graph(models.GraphModel):
             node.isrequired = nodeobj.get("isrequired", False)
             node.exportable = nodeobj.get("exportable", False)
             node.fieldname = nodeobj.get("fieldname", "")
+            self.create_node_alias(node)
 
             node.nodeid = uuid.UUID(str(node.nodeid))
 
@@ -335,10 +336,9 @@ class Graph(models.GraphModel):
                 old.update({k: v})
         return old, new
 
-    def _update_node(self, node, datatype_factory, se):
+    def update_es_node_mapping(self, node, datatype_factory, se):
         already_saved = models.Node.objects.filter(pk=node.nodeid).exists()
         saved_node_datatype = None
-        node.save()
         if already_saved:
             saved_node = models.Node.objects.get(pk=node.nodeid)
             saved_node_datatype = saved_node.datatype
@@ -371,10 +371,13 @@ class Graph(models.GraphModel):
 
             if nodeid is not None:
                 node = self.nodes[nodeid]
-                self._update_node(node, datatype_factory, se)
+                self.update_es_node_mapping(node, datatype_factory, se)
+                self.create_node_alias(node)
+                node.save()
             else:
                 for node in self.nodes.values():
-                    self._update_node(node, datatype_factory, se)
+                    self.update_es_node_mapping(node, datatype_factory, se)
+                    node.save()
 
             for edge in self.edges.values():
                 edge.save()
@@ -538,9 +541,9 @@ class Graph(models.GraphModel):
 
             return branch_copy
 
-    def make_name_unique(self, name, names_to_check):
+    def make_name_unique(self, name, names_to_check, suffix_delimiter="_"):
         """
-        Makes a name unique among a list of name
+        Makes a name unique among a list of names
 
         Arguments:
         name -- the name to check and modfiy to make unique in the list of "names_to_check"
@@ -550,7 +553,7 @@ class Graph(models.GraphModel):
         i = 1
         temp_node_name = name
         while temp_node_name in names_to_check:
-            temp_node_name = "{0}_{1}".format(name, i)
+            temp_node_name = "{0}{1}{2}".format(name, suffix_delimiter, i)
             i += 1
         return temp_node_name
 
@@ -689,6 +692,10 @@ class Graph(models.GraphModel):
         # returns a list of node ids sorted by nodes that are collector nodes first and then others last
         node_ids = sorted(copy_of_self.nodes, key=lambda node_id: copy_of_self.nodes[node_id].is_collector, reverse=True)
 
+        for nodeid, node in copy_of_self.nodes.items():
+            if node.datatype == "geojson-feature-collection":
+                node.config["advancedStyle"] = ""
+                node.config["advancedStyling"] = False
         copy_of_self.pk = uuid.uuid1()
         node_map = {}
         card_map = {}
@@ -859,9 +866,9 @@ class Graph(models.GraphModel):
             if self.is_editable() is False and tile_count > 0:
                 raise GraphValidationError(
                     _(
-                        f"Your resource model: {self.name}, already has instances saved. \
+                        "Your resource model: {self.name}, already has instances saved. \
                             You cannot delete nodes from a Resource Model with instances."
-                    ),
+                    ).format(**locals()),
                     1006,
                 )
 
@@ -1345,9 +1352,9 @@ class Graph(models.GraphModel):
                 if len(unpermitted_edits) > 0:
                     raise GraphValidationError(
                         _(
-                            f"Your resource model: {self.name}, already has instances saved. \
+                            "Your resource model: {self.name}, already has instances saved. \
                                 You cannot modify a Resource Model with instances."
-                        ),
+                        ).format(**locals()),
                         1006,
                     )
 
@@ -1367,14 +1374,27 @@ class Graph(models.GraphModel):
                 message = _('Duplicate node name: "{0}". All node names in a card must be unique.'.format(node.name))
                 raise GraphValidationError(message)
             elif node.is_editable() is False:
-                message = "The name of this node cannot be changed because business data has already been saved to a card that this node is part of."
-                raise GraphValidationError(_(message))
+                if node.name != models.Node.objects.values_list("name", flat=True).get(pk=node.nodeid):
+                    message = "The name of this node cannot be changed because business data has already been saved to a card that this node is part of."
+                    raise GraphValidationError(_(message))
             else:
                 sibling_node_names = [node.name for node in self.get_sibling_nodes(node)]
                 if node.name in sibling_node_names:
                     message = _('Duplicate node name: "{0}". All sibling node names must be unique.'.format(node.name))
                     raise GraphValidationError(message)
 
+    def create_node_alias(self, node):
+        """
+        Assigns a unique, slugified version of a node's name as that node's alias.
+        """
+
+        with connection.cursor() as cursor:
+            cursor.callproc("__arches_slugify", [node.name])
+            row = cursor.fetchone()
+            aliases = [n.alias for n in self.nodes.values() if node.alias != n.alias]
+            node.alias = self.make_name_unique(row[0], aliases, "_n")
+        return node.alias
+
     def validate(self):
         """
         validates certain aspects of resource graphs according to defined rules:
@@ -1393,9 +1413,9 @@ class Graph(models.GraphModel):
             if self.root.is_collector is True:
                 raise GraphValidationError(
                     _(
-                        f"The top node of your resource graph: {self.root.name} needs to be a collector. \
+                        "The top node of your resource graph: {self.root.name} needs to be a collector. \
                             Hint: check that nodegroup_id of your resource node(s) are not null."
-                    ),
+                    ).format(**locals()),
                     997,
                 )
             if self.root.datatype != "semantic":
@@ -1418,7 +1438,9 @@ class Graph(models.GraphModel):
                 fieldname = fieldname[:10]
             try:
                 dupe = fieldnames[fieldname]
-                raise GraphValidationError(_(f"Field name must be unique to the graph; '{fieldname}' already exists."), 1009)
+                raise GraphValidationError(
+                    _("Field name must be unique to the graph; '{fieldname}' already exists.").format(**locals()), 1009
+                )
             except KeyError:
                 fieldnames[fieldname] = True
 
@@ -1450,10 +1472,10 @@ class Graph(models.GraphModel):
                 if edge.ontologyproperty is None:
                     raise GraphValidationError(
                         _(
-                            f"You must specify an ontology property. Your graph isn't semantically valid. \
+                            "You must specify an ontology property. Your graph isn't semantically valid. \
                                 Entity domain '{edge.domainnode.ontologyclass}' and \
                                 Entity range '{edge.rangenode.ontologyclass}' can not be related via Property '{edge.ontologyproperty}'."
-                        ),
+                        ).format(**locals()),
                         1002,
                     )
                 property_found = False
@@ -1469,10 +1491,10 @@ class Graph(models.GraphModel):
                 if not okay:
                     raise GraphValidationError(
                         _(
-                            f"Your graph isn't semantically valid. Entity domain '{edge.domainnode.ontologyclass}' and \
+                            "Your graph isn't semantically valid. Entity domain '{edge.domainnode.ontologyclass}' and \
                                 Entity range '{edge.rangenode.ontologyclass}' cannot \
                                 be related via Property '{edge.ontologyproperty}'."
-                        ),
+                        ).format(**locals()),
                         1003,
                     )
                 elif not property_found:
@@ -1512,7 +1534,7 @@ class Graph(models.GraphModel):
         if self.slug is not None:
             graphs_with_matching_slug = models.GraphModel.objects.exclude(slug__isnull=True).filter(slug=self.slug)
             if graphs_with_matching_slug.exists() and graphs_with_matching_slug[0].graphid != self.graphid:
-                raise GraphValidationError(_(f"Another resource modal already uses the slug '{self.slug}'"), 1007)
+                raise GraphValidationError(_("Another resource modal already uses the slug '{self.slug}'").format(**locals()), 1007)
 
 
 class GraphValidationError(Exception):
