@@ -34,8 +34,11 @@ from arches.app.models.concept import get_preflabel_from_valueid
 from arches.app.models.system_settings import settings
 from arches.app.search.search_engine_factory import SearchEngineInstance as se
 from arches.app.search.mappings import TERMS_INDEX, RESOURCE_RELATIONS_INDEX, RESOURCES_INDEX
-from arches.app.search.elasticsearch_dsl_builder import Query, Bool, Terms
-from arches.app.utils import import_class_from_string
+from arches.app.search.elasticsearch_dsl_builder import Query, Bool, Terms, Nested
+from arches.app.tasks import index_resource
+from arches.app.utils import import_class_from_string, task_management
+from arches.app.utils.label_based_graph import LabelBasedGraph
+from arches.app.utils.label_based_graph_v2 import LabelBasedGraph as LabelBasedGraphV2
 from guardian.shortcuts import assign_perm, remove_perm
 from guardian.exceptions import NotUserNorGroup
 from arches.app.utils.betterJSONSerializer import JSONSerializer, JSONDeserializer
@@ -67,28 +70,28 @@ class Resource(models.ResourceInstance):
         # end from models.ResourceInstance
         self.tiles = []
 
-    def get_descriptor(self, descriptor):
-        module = importlib.import_module("arches.app.functions.primary_descriptors")
-        PrimaryDescriptorsFunction = getattr(module, "PrimaryDescriptorsFunction")()
-        functionConfig = models.FunctionXGraph.objects.filter(graph_id=self.graph_id, function__functiontype="primarydescriptors")
-        if len(functionConfig) == 1:
-            return PrimaryDescriptorsFunction.get_primary_descriptor_from_nodes(self, functionConfig[0].config[descriptor])
+    def get_descriptor(self, descriptor, context):
+        graph_function = models.FunctionXGraph.objects.filter(
+            graph_id=self.graph_id, function__functiontype="primarydescriptors"
+        ).select_related("function")
+        if len(graph_function) == 1:
+            module = graph_function[0].function.get_class_module()()
+            return module.get_primary_descriptor_from_nodes(
+                self, graph_function[0].config["descriptor_types"][descriptor], context
+            )
         else:
             return "undefined"
 
-    @property
-    def displaydescription(self):
-        return self.get_descriptor("description")
+    def displaydescription(self, context=None):
+        return self.get_descriptor("description", context)
 
-    @property
-    def map_popup(self):
-        return self.get_descriptor("map_popup")
+    def map_popup(self, context=None):
+        return self.get_descriptor("map_popup", context)
 
-    @property
-    def displayname(self):
-        return self.get_descriptor("name")
+    def displayname(self, context=None):
+        return self.get_descriptor("name", context)
 
-    def save_edit(self, user={}, note="", edit_type=""):
+    def save_edit(self, user={}, note="", edit_type="", transaction_id=None):
         timestamp = datetime.datetime.now()
         edit = EditLog()
         edit.resourceclassid = self.graph_id
@@ -99,6 +102,8 @@ class Resource(models.ResourceInstance):
         edit.user_lastname = getattr(user, "last_name", "")
         edit.note = note
         edit.timestamp = timestamp
+        if transaction_id is not None:
+            edit.transactionid = transaction_id
         edit.edittype = edit_type
         edit.save()
 
@@ -106,6 +111,11 @@ class Resource(models.ResourceInstance):
         """
         Saves and indexes a single resource
 
+        Keyword Arguments:
+        request -- the request object
+        user -- the user to associate the edit with if the user can't be derived from the request
+        index -- True(default) to index the resource, otherwise don't index the resource
+
         """
         graph = models.GraphModel.objects.get(graphid=self.graph_id)
         if graph.isactive is False:
@@ -113,10 +123,13 @@ class Resource(models.ResourceInstance):
             raise ModelInactiveError(message)
         request = kwargs.pop("request", None)
         user = kwargs.pop("user", None)
+        index = kwargs.pop("index", True)
+        context = kwargs.pop("context", None)
+        transaction_id = kwargs.pop("transaction_id", None)
         super(Resource, self).save(*args, **kwargs)
         for tile in self.tiles:
             tile.resourceinstance_id = self.resourceinstanceid
-            saved_tile = tile.save(request=request, index=False)
+            tile.save(request=request, index=False, transaction_id=transaction_id, context=context)
         if request is None:
             if user is None:
                 user = {}
@@ -129,8 +142,9 @@ class Resource(models.ResourceInstance):
         except NotUserNorGroup:
             pass
 
-        self.save_edit(user=user, edit_type="create")
-        self.index()
+        self.save_edit(user=user, edit_type="create", transaction_id=transaction_id)
+        if index is True:
+            self.index(context)
 
     def get_root_ontology(self):
         """
@@ -144,13 +158,15 @@ class Resource(models.ResourceInstance):
 
         return root_ontology_class
 
-    def load_tiles(self):
+    def load_tiles(self, user=None, perm=None):
         """
         Loads the resource's tiles array with all the tiles from the database as a flat list
 
         """
 
         self.tiles = list(models.TileModel.objects.filter(resourceinstance=self))
+        if user:
+            self.tiles = [tile for tile in self.tiles if tile.nodegroup_id is not None and user.has_perm(perm, tile.nodegroup)]
 
     # # flatten out the nested tiles into a single array
     def get_flattened_tiles(self):
@@ -160,13 +176,16 @@ class Resource(models.ResourceInstance):
         return tiles
 
     @staticmethod
-    def bulk_save(resources):
+    def bulk_save(resources, transaction_id=None):
         """
         Saves and indexes a list of resources
 
         Arguments:
         resources -- a list of resource models
 
+        Keyword Arguments:
+        transaction_id -- a uuid identifing the save of these instances as belonging to a collective load or process
+
         """
 
         datatype_factory = DataTypeFactory()
@@ -188,9 +207,11 @@ class Resource(models.ResourceInstance):
 
         start = time()
         for resource in resources:
-            resource.save_edit(edit_type="create")
+            resource.save_edit(edit_type="create", transaction_id=transaction_id)
 
-        resources[0].tiles[0].save_edit(note=f"Bulk created: {len(tiles)} for {len(resources)} resources.", edit_type="bulk_create")
+        resources[0].tiles[0].save_edit(
+            note=f"Bulk created: {len(tiles)} for {len(resources)} resources.", edit_type="bulk_create", transaction_id=transaction_id
+        )
 
         print("Time to save resource edits: %s" % datetime.timedelta(seconds=time() - start))
 
@@ -208,28 +229,36 @@ class Resource(models.ResourceInstance):
         se.bulk_index(documents)
         se.bulk_index(term_list)
 
-    def index(self):
+    def index(self, context=None):
         """
         Indexes all the nessesary items values of a resource to support search
 
+        Keyword Arguments:
+        context -- a string such as "copy" to indicate conditions under which a document is indexed
+
         """
 
         if str(self.graph_id) != str(settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID):
             datatype_factory = DataTypeFactory()
             node_datatypes = {str(nodeid): datatype for nodeid, datatype in models.Node.objects.values_list("nodeid", "datatype")}
-            document, terms = self.get_documents_to_index(datatype_factory=datatype_factory, node_datatypes=node_datatypes)
+            document, terms = self.get_documents_to_index(datatype_factory=datatype_factory, node_datatypes=node_datatypes, context=context)
             document["root_ontology_class"] = self.get_root_ontology()
             doc = JSONSerializer().serializeToPython(document)
             se.index_data(index=RESOURCES_INDEX, body=doc, id=self.pk)
             for term in terms:
                 se.index_data("terms", body=term["_source"], id=term["_id"])
 
-            for index in settings.ELASTICSEARCH_CUSTOM_INDEXES:
-                es_index = import_class_from_string(index["module"])(index["name"])
-                document, doc_id = es_index.get_documents_to_index(self, document["tiles"])
-                es_index.index_document(document=document, id=doc_id)
+            celery_worker_running = task_management.check_if_celery_available()
 
-    def get_documents_to_index(self, fetchTiles=True, datatype_factory=None, node_datatypes=None):
+            for index in settings.ELASTICSEARCH_CUSTOM_INDEXES:
+                if celery_worker_running and index.get("should_update_asynchronously"):
+                    index_resource.apply_async([index["module"], index["name"], self.pk, [tile.pk for tile in document["tiles"]]])
+                else:
+                    es_index = import_class_from_string(index["module"])(index["name"])
+                    doc, doc_id = es_index.get_documents_to_index(self, document["tiles"])
+                    es_index.index_document(document=doc, id=doc_id)
+
+    def get_documents_to_index(self, fetchTiles=True, datatype_factory=None, node_datatypes=None, context=None):
         """
         Gets all the documents nessesary to index a single resource
         returns a tuple of a document and list of terms
@@ -238,6 +267,7 @@ class Resource(models.ResourceInstance):
         fetchTiles -- instead of fetching the tiles from the database get them off the model itself
         datatype_factory -- refernce to the DataTypeFactory instance
         node_datatypes -- a dictionary of datatypes keyed to node ids
+        context -- a string such as "copy" to indicate conditions under which a document is indexed
 
         """
 
@@ -249,9 +279,9 @@ class Resource(models.ResourceInstance):
         document["displayname"] = None
         document["root_ontology_class"] = self.get_root_ontology()
         document["legacyid"] = self.legacyid
-        document["displayname"] = self.displayname
-        document["displaydescription"] = self.displaydescription
-        document["map_popup"] = self.map_popup
+        document["displayname"] = self.displayname(context)
+        document["displaydescription"] = self.displaydescription(context)
+        document["map_popup"] = self.map_popup(context)
 
         tiles = list(models.TileModel.objects.filter(resourceinstance=self)) if fetchTiles else self.tiles
 
@@ -275,8 +305,8 @@ class Resource(models.ResourceInstance):
 
         for tile in document["tiles"]:
             for nodeid, nodevalue in tile.data.items():
-                datatype = node_datatypes[nodeid]
                 if nodevalue != "" and nodevalue != [] and nodevalue != {} and nodevalue is not None:
+                    datatype = node_datatypes[nodeid]
                     datatype_instance = datatype_factory.get_instance(datatype)
                     datatype_instance.append_to_document(document, nodevalue, nodeid, tile)
                     node_terms = datatype_instance.get_search_terms(nodevalue, nodeid)
@@ -303,8 +333,8 @@ class Resource(models.ResourceInstance):
                     for user, edit in provisionaledits.items():
                         if edit["status"] == "review":
                             for nodeid, nodevalue in edit["value"].items():
-                                datatype = node_datatypes[nodeid]
                                 if nodevalue != "" and nodevalue != [] and nodevalue != {} and nodevalue is not None:
+                                    datatype = node_datatypes[nodeid]
                                     datatype_instance = datatype_factory.get_instance(datatype)
                                     datatype_instance.append_to_document(document, nodevalue, nodeid, tile, True)
                                     node_terms = datatype_instance.get_search_terms(nodevalue, nodeid)
@@ -325,12 +355,16 @@ class Resource(models.ResourceInstance):
 
         return document, terms
 
-    def delete(self, user={}, note=""):
+    def delete(self, user={}, index=True, transaction_id=None):
         """
         Deletes a single resource and any related indexed data
 
         """
 
+        # note that deferring index will require:
+        # - that any resources related to the to-be-deleted resource get re-indexed
+        # - that the index for the to-be-deleted resource gets deleted
+
         permit_deletion = False
         graph = models.GraphModel.objects.get(graphid=self.graph_id)
         if graph.isactive is False:
@@ -349,53 +383,114 @@ class Resource(models.ResourceInstance):
             permit_deletion = True
 
         if permit_deletion is True:
-            related_resources = self.get_related_resources(lang="en-US", start=0, limit=1000, page=0)
-            for rr in related_resources["resource_relationships"]:
-                # delete any related resource entries, also reindex the resource that references this resource that's being deleted
-                try:
-                    resourceXresource = models.ResourceXResource.objects.get(pk=rr["resourcexid"])
-                    resource_to_reindex = (
-                        resourceXresource.resourceinstanceidfrom_id
-                        if resourceXresource.resourceinstanceidto_id == self.resourceinstanceid
-                        else resourceXresource.resourceinstanceidto_id
-                    )
-                    resourceXresource.delete(deletedResourceId=self.resourceinstanceid)
-                    res = Resource.objects.get(pk=resource_to_reindex)
-                    res.load_tiles()
-                    res.index()
-                except ObjectDoesNotExist:
-                    se.delete(index=RESOURCE_RELATIONS_INDEX, id=rr["resourcexid"])
+            for related_resource in models.ResourceXResource.objects.filter(
+                Q(resourceinstanceidfrom=self.resourceinstanceid) | Q(resourceinstanceidto=self.resourceinstanceid)
+            ):
+                related_resource.delete(deletedResourceId=self.resourceinstanceid, index=False)
 
-            query = Query(se)
-            bool_query = Bool()
-            bool_query.filter(Terms(field="resourceinstanceid", terms=[self.resourceinstanceid]))
-            query.add_query(bool_query)
-            results = query.search(index=TERMS_INDEX)["hits"]["hits"]
-            for result in results:
-                se.delete(index=TERMS_INDEX, id=result["_id"])
-            se.delete(index=RESOURCES_INDEX, id=self.resourceinstanceid)
+            if index:
+                self.delete_index()
 
             try:
-                self.save_edit(edit_type="delete", user=user, note=self.displayname)
+                self.save_edit(edit_type="delete", user=user, note=self.displayname(), transaction_id=transaction_id)
             except:
                 pass
             super(Resource, self).delete()
 
         return permit_deletion
 
+    def delete_index(self, resourceinstanceid=None):
+        """
+        Deletes all references to a resource from all indexes
+
+        Keyword Arguments:
+        resourceinstanceid -- the resource instance id to delete from related indexes, if supplied will use this over self.resourceinstanceid
+        """
+
+        if resourceinstanceid is None:
+            resourceinstanceid = self.resourceinstanceid
+        resourceinstanceid = str(resourceinstanceid)
+
+        # delete any related terms
+        query = Query(se)
+        bool_query = Bool()
+        bool_query.filter(Terms(field="resourceinstanceid", terms=[resourceinstanceid]))
+        query.add_query(bool_query)
+        query.delete(index=TERMS_INDEX)
+
+        # delete any related resource index entries
+        query = Query(se)
+        bool_query = Bool()
+        bool_query.should(Terms(field="resourceinstanceidto", terms=[resourceinstanceid]))
+        bool_query.should(Terms(field="resourceinstanceidfrom", terms=[resourceinstanceid]))
+        query.add_query(bool_query)
+        query.delete(index=RESOURCE_RELATIONS_INDEX)
+
+        # reindex any related resources
+        query = Query(se)
+        bool_query = Bool()
+        bool_query.filter(Nested(path="ids", query=Terms(field="ids.id", terms=[resourceinstanceid])))
+        query.add_query(bool_query)
+        results = query.search(index=RESOURCES_INDEX)["hits"]["hits"]
+        for result in results:
+            try:
+                res = Resource.objects.get(pk=result["_id"])
+                res.load_tiles()
+                res.index()
+            except ObjectDoesNotExist:
+                pass
+
+        # delete resource index
+        se.delete(index=RESOURCES_INDEX, id=resourceinstanceid)
+
+        # delete resources from custom indexes
+        for index in settings.ELASTICSEARCH_CUSTOM_INDEXES:
+            es_index = import_class_from_string(index["module"])(index["name"])
+            es_index.delete_resources(resources=self)
+
+    def validate(self, verbose=False, strict=False):
+        """
+        Keyword Arguments:
+        verbose -- False(default) to only show the first error thrown in any tile, True to show all the errors in all the tiles
+        strict -- False(default), True to use a more complete check on the datatype
+            (eg: check for the existance of a referenced resoure on the resource-instance datatype)
+        """
+
+        from arches.app.models.tile import Tile, TileValidationError
+
+        errors = []
+        tiles = self.tiles
+        if len(self.tiles) == 0:
+            tiles = Tile.objects.filter(resourceinstance=self)
+
+        for tile in tiles:
+            try:
+                tile.validate(raise_early=(not verbose), strict=strict)
+            except TileValidationError as err:
+                errors += err.message if isinstance(err.message, list) else [err.message]
+        return errors
+
     def get_related_resources(
-        self, lang="en-US", limit=settings.RELATED_RESOURCES_EXPORT_LIMIT, start=0, page=0, user=None, resourceinstance_graphid=None,
+        self,
+        lang="en-US",
+        limit=settings.RELATED_RESOURCES_EXPORT_LIMIT,
+        start=0,
+        page=0,
+        user=None,
+        resourceinstance_graphid=None,
+        graphs=None,
     ):
         """
         Returns an object that lists the related resources, the relationship types, and a reference to the current resource
 
         """
-        graphs = (
-            models.GraphModel.objects.all()
-            .exclude(pk=settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID)
-            .exclude(isresource=False)
-            .exclude(isactive=False)
-        )
+        if not graphs:
+            graphs = list(
+                models.GraphModel.objects.all()
+                .exclude(pk=settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID)
+                .exclude(isresource=False)
+                .exclude(isactive=False)
+            )
 
         graph_lookup = {
             str(graph.graphid): {"name": graph.name, "iconclass": graph.iconclass, "fillColor": graph.color} for graph in graphs
@@ -414,10 +509,17 @@ class Resource(models.ResourceInstance):
             bool_filter.should(Terms(field="resourceinstanceidto", terms=resourceinstanceid))
 
             if resourceinstance_graphid:
-                graph_id_filter = Bool()
-                graph_id_filter.should(Terms(field="resourceinstancefrom_graphid", terms=resourceinstance_graphid))
-                graph_id_filter.should(Terms(field="resourceinstanceto_graphid", terms=resourceinstance_graphid))
-                bool_filter.must(graph_id_filter)
+                graph_filter = Bool()
+                to_graph_id_filter = Bool()
+                to_graph_id_filter.filter(Terms(field="resourceinstancefrom_graphid", terms=str(self.graph_id)))
+                to_graph_id_filter.filter(Terms(field="resourceinstanceto_graphid", terms=resourceinstance_graphid))
+                graph_filter.should(to_graph_id_filter)
+
+                from_graph_id_filter = Bool()
+                from_graph_id_filter.filter(Terms(field="resourceinstancefrom_graphid", terms=resourceinstance_graphid))
+                from_graph_id_filter.filter(Terms(field="resourceinstanceto_graphid", terms=str(self.graph_id)))
+                graph_filter.should(from_graph_id_filter)
+                bool_filter.must(graph_filter)
 
             query.add_query(bool_filter)
 
@@ -427,6 +529,8 @@ class Resource(models.ResourceInstance):
             resourceinstanceid=self.resourceinstanceid, start=start, limit=limit, resourceinstance_graphid=resourceinstance_graphid,
         )
 
+
+
         ret["total"] = resource_relations["hits"]["total"]
         instanceids = set()
 
@@ -454,7 +558,11 @@ class Resource(models.ResourceInstance):
             related_resources = se.search(index=RESOURCES_INDEX, id=list(instanceids))
             if related_resources:
                 for resource in related_resources["docs"]:
-                    relations = get_relations(resourceinstanceid=resource["_id"], start=0, limit=0,)
+                    relations = get_relations(
+                        resourceinstanceid=resource["_id"],
+                        start=0,
+                        limit=0,
+                    )
                     if resource["found"]:
                         resource["_source"]["total_relations"] = relations["hits"]["total"]
                         ret["related_resources"].append(resource["_source"])
@@ -463,7 +571,7 @@ class Resource(models.ResourceInstance):
 
     def copy(self):
         """
-        Returns a copy of this resource instance includeing a copy of all tiles associated with this resource instance
+        Returns a copy of this resource instance including a copy of all tiles associated with this resource instance
 
         """
         # need this here to prevent a circular import error
@@ -492,7 +600,7 @@ class Resource(models.ResourceInstance):
                 tile.parenttile = id_map[tile.parenttile_id]
 
         with transaction.atomic():
-            new_resource.save()
+            new_resource.save(context="copy")
 
         return new_resource
 
@@ -506,10 +614,44 @@ class Resource(models.ResourceInstance):
         """
 
         ret = JSONSerializer().handle_model(self)
+        ret["displayname"] = self.displayname()
         ret["tiles"] = self.tiles
 
         return JSONSerializer().serializeToPython(ret)
 
+    def to_json(self, compact=True, hide_empty_nodes=False, user=None, perm=None, version=None, hide_hidden_nodes=False):
+        """
+        Returns resource represented as disambiguated JSON graph
+
+        Keyword Arguments:
+        compact -- type bool: hide superfluous node data
+        hide_empty_nodes -- type bool: hide nodes without data
+        """
+        if version is None:
+            return LabelBasedGraph.from_resource(
+                resource=self, compact=compact, hide_empty_nodes=hide_empty_nodes, user=user, perm=perm, hide_hidden_nodes=hide_hidden_nodes
+            )
+        elif version == "beta":
+            return LabelBasedGraphV2.from_resource(
+                resource=self, compact=compact, hide_empty_nodes=hide_empty_nodes, user=user, perm=perm, hide_hidden_nodes=hide_hidden_nodes
+            )
+
+    @staticmethod
+    def to_json__bulk(resources, compact=True, hide_empty_nodes=False, version=None):
+        """
+        Returns list of resources represented as disambiguated JSON graphs
+
+        Keyword Arguments:
+        resources -- list of Resource
+        compact -- type bool: hide superfluous node data
+        hide_empty_nodes -- type bool: hide nodes without data
+        """
+
+        if version is None:
+            return LabelBasedGraph.from_resources(resources=resources, compact=compact, hide_empty_nodes=hide_empty_nodes)
+        elif version == "beta":
+            return LabelBasedGraphV2.from_resources(resources=resources, compact=compact, hide_empty_nodes=hide_empty_nodes)
+
     def get_node_values(self, node_name):
         """
         Take a node_name (string) as an argument and return a list of values.
