@@ -116,7 +116,25 @@ class Tile(models.TileModel):
         newprovisionalvalue=None,
         oldprovisionalvalue=None,
         provisional_edit_log_details=None,
+        transaction_id=None,
+        new_resource_created=False,
     ):
+        if new_resource_created:
+            timestamp = datetime.datetime.now()
+            resource_edit = EditLog()
+            resource_edit.resourceclassid = self.resourceinstance.graph_id
+            resource_edit.resourceinstanceid = self.resourceinstance.resourceinstanceid
+            resource_edit.edittype = "create"
+            resource_edit.timestamp = timestamp
+            resource_edit.userid = getattr(user, "id", "")
+            resource_edit.user_email = getattr(user, "email", "")
+            resource_edit.user_firstname = getattr(user, "first_name", "")
+            resource_edit.user_lastname = getattr(user, "last_name", "")
+            resource_edit.user_username = getattr(user, "username", "")
+            if transaction_id is not None:
+                resource_edit.transactionid = transaction_id
+            resource_edit.save()
+
         timestamp = datetime.datetime.now()
         edit = EditLog()
         edit.resourceclassid = self.resourceinstance.graph_id
@@ -133,13 +151,15 @@ class Tile(models.TileModel):
         edit.user_firstname = getattr(user, "first_name", "")
         edit.user_lastname = getattr(user, "last_name", "")
         edit.user_username = getattr(user, "username", "")
-        edit.resourcedisplayname = Resource.objects.get(resourceinstanceid=self.resourceinstance.resourceinstanceid).displayname
+        edit.resourcedisplayname = Resource.objects.get(resourceinstanceid=self.resourceinstance.resourceinstanceid).displayname()
         edit.oldvalue = old_value
         edit.newvalue = new_value
         edit.timestamp = timestamp
         edit.edittype = edit_type
         edit.newprovisionalvalue = newprovisionalvalue
         edit.oldprovisionalvalue = oldprovisionalvalue
+        if transaction_id is not None:
+            edit.transactionid = transaction_id
         edit.save()
 
     def tile_collects_data(self):
@@ -211,7 +231,9 @@ class Tile(models.TileModel):
                 edit = edits[str(user.id)]
         return edit
 
-    def check_for_constraint_violation(self, request):
+    def check_for_constraint_violation(self):
+        if settings.BYPASS_UNIQUE_CONSTRAINT_TILE_VALIDATION:
+            return
         card = models.CardModel.objects.get(nodegroup=self.nodegroup)
         constraints = models.ConstraintModel.objects.filter(card=card)
         if constraints.count() > 0:
@@ -230,7 +252,20 @@ class Tile(models.TileModel):
                         for node in nodes:
                             datatype = self.datatype_factory.get_instance(node.datatype)
                             nodeid = str(node.nodeid)
-                            if datatype.values_match(tile.data[nodeid], self.data[nodeid]):
+                            tile_data = ""
+                            if tile.provisionaledits is None:
+                                # If this is not a provisional tile, the data should
+                                # exist, so we check it normally
+                                tile_data = tile.data[nodeid]
+                            else:
+                                # If it is a provisional tile, we need to check the
+                                # provisional edits for clashing values
+                                for edit_id in tile.provisionaledits.keys():
+                                    edit_data = tile.provisionaledits[str(edit_id)]
+                                    if nodeid in edit_data["value"]:
+                                        tile_data = edit_data["value"][nodeid]
+                                        break
+                            if datatype.values_match(tile_data, self.data[nodeid]):
                                 match = True
                                 duplicate_values.append(datatype.get_display_value(tile, node))
                             else:
@@ -243,40 +278,56 @@ class Tile(models.TileModel):
                             )
                             raise TileValidationError(message + (", ").join(duplicate_values))
 
-    def check_for_missing_nodes(self, request):
+    def check_for_missing_nodes(self):
+        if settings.BYPASS_REQUIRED_VALUE_TILE_VALIDATION:
+            return
         missing_nodes = []
         for nodeid, value in self.data.items():
             try:
                 node = models.Node.objects.get(nodeid=nodeid)
                 datatype = self.datatype_factory.get_instance(node.datatype)
                 datatype.clean(self, nodeid)
-                if request is not None:
-                    if self.data[nodeid] is None and node.isrequired is True:
-                        if len(node.cardxnodexwidget_set.all()) > 0:
-                            missing_nodes.append(node.cardxnodexwidget_set.all()[0].label)
-                        else:
-                            missing_nodes.append(node.name)
-            except Exception as e:
+                if self.data[nodeid] is None and node.isrequired is True:
+                    if len(node.cardxnodexwidget_set.all()) > 0:
+                        missing_nodes.append(node.cardxnodexwidget_set.all()[0].label)
+                    else:
+                        missing_nodes.append(node.name)
+            except Exception:
                 warning = _(
-                    f"Error checking for missing node. Nodeid: {nodeid} with value: {value}, not in nodes. \
+                    "Error checking for missing node. Nodeid: {nodeid} with value: {value}, not in nodes. \
                     You may have a node in your business data that no longer exists in any graphs."
-                )
+                ).format(**locals())
                 logger.warning(warning)
         if missing_nodes != []:
             message = _("This card requires values for the following: ")
             message += (", ").join(missing_nodes)
             raise TileValidationError(message)
 
-    def validate(self, errors=None):
+    def validate(self, errors=None, raise_early=True, strict=False, request=None):
+        """
+        Keyword Arguments:
+        errors -- supply and list to have errors appened on to
+        raise_early -- True(default) to raise an error on the first value in the tile that throws an error
+            otherwise throw an error only after all nodes in a tile have been validated
+        strict -- False(default), True to use a more complete check on the datatype
+            (eg: check for the existance of a referenced resoure on the resource-instance datatype)
+        """
+
+        tile_errors = []
+
         for nodeid, value in self.data.items():
             node = models.Node.objects.get(nodeid=nodeid)
             datatype = self.datatype_factory.get_instance(node.datatype)
-            error = datatype.validate(value, node=node)
+            error = datatype.validate(value, node=node, strict=strict, request=request)
+            tile_errors += error
             for error_instance in error:
                 if error_instance["type"] == "ERROR":
-                    raise TileValidationError(_("{0}".format(error_instance["message"])))
+                    if raise_early:
+                        raise TileValidationError(_("{0}".format(error_instance["message"])))
             if errors is not None:
                 errors += error
+        if not raise_early:
+            raise TileValidationError(tile_errors)
         return errors
 
     def get_tile_data(self, user_id=None):
@@ -308,9 +359,10 @@ class Tile(models.TileModel):
         request = kwargs.pop("request", None)
         index = kwargs.pop("index", True)
         user = kwargs.pop("user", None)
-        log = kwargs.pop("log", True)
+        new_resource_created = kwargs.pop("new_resource_created", False)
+        context = kwargs.pop("context", None)
+        transaction_id = kwargs.pop("transaction_id", None)
         provisional_edit_log_details = kwargs.pop("provisional_edit_log_details", None)
-        missing_nodes = []
         creating_new_tile = True
         user_is_reviewer = False
         newprovisionalvalue = None
@@ -323,24 +375,22 @@ class Tile(models.TileModel):
         except AttributeError:  # no user - probably importing data
             user = None
 
-
-        # if user is not None:
-        #     self.validate([])
-
         with transaction.atomic():
-            for nodeid, value in self.data.items():
+            for nodeid in self.data.keys():
                 node = models.Node.objects.get(nodeid=nodeid)
                 datatype = self.datatype_factory.get_instance(node.datatype)
                 datatype.pre_tile_save(self, nodeid)
-            self.__preSave(request)
-            self.check_for_missing_nodes(request)
-            self.check_for_constraint_violation(request)
+            self.__preSave(request, context=context)
+            self.check_for_missing_nodes()
+            self.check_for_constraint_violation()
 
             creating_new_tile = models.TileModel.objects.filter(pk=self.tileid).exists() is False
             edit_type = "tile create" if (creating_new_tile is True) else "tile edit"
 
             if creating_new_tile is False:
                 existing_model = models.TileModel.objects.get(pk=self.tileid)
+            else:
+                self.populate_missing_nodes()
 
             # this section moves the data over from self.data to self.provisionaledits if certain users permissions are in force
             # then self.data is restored from the previously saved tile data
@@ -368,14 +418,14 @@ class Tile(models.TileModel):
                     }
 
             if user is not None:
-                self.validate([])
+                self.validate([], request=request)
 
             super(Tile, self).save(*args, **kwargs)
             # We have to save the edit log record after calling save so that the
             # resource's displayname changes are avaliable
             user = {} if user is None else user
             self.datatype_post_save_actions(request)
-            self.__postSave(request)
+            self.__postSave(request, context=context)
             if creating_new_tile is True:
                 self.save_edit(
                     user=user,
@@ -384,6 +434,8 @@ class Tile(models.TileModel):
                     new_value=self.data,
                     newprovisionalvalue=newprovisionalvalue,
                     provisional_edit_log_details=provisional_edit_log_details,
+                    transaction_id=transaction_id,
+                    new_resource_created=new_resource_created,
                 )
             else:
                 self.save_edit(
@@ -394,6 +446,7 @@ class Tile(models.TileModel):
                     newprovisionalvalue=newprovisionalvalue,
                     oldprovisionalvalue=oldprovisionalvalue,
                     provisional_edit_log_details=provisional_edit_log_details,
+                    transaction_id=transaction_id,
                 )
 
             if index:
@@ -404,9 +457,18 @@ class Tile(models.TileModel):
                 tile.parenttile = self
                 tile.save(*args, request=request, index=index, **kwargs)
 
+    def populate_missing_nodes(self):
+        first_node = next(iter(self.data.items()), None)
+        if first_node is not None:
+            result = Tile.get_blank_tile_from_nodegroup_id(nodegroup_id=self.nodegroup_id)
+            result.data.update(self.data)
+            self.data = result.data
+
     def delete(self, *args, **kwargs):
         se = SearchEngineFactory().create()
         request = kwargs.pop("request", None)
+        index = kwargs.pop("index", True)
+        transaction_id = kwargs.pop("transaction_id", None)
         provisional_edit_log_details = kwargs.pop("provisional_edit_log_details", None)
         for tile in self.tiles:
             tile.delete(*args, request=request, **kwargs)
@@ -418,29 +480,31 @@ class Tile(models.TileModel):
             user_is_reviewer = True
 
         if user_is_reviewer is True or self.user_owns_provisional(user):
-            query = Query(se)
-            bool_query = Bool()
-            bool_query.filter(Terms(field="tileid", terms=[self.tileid]))
-            query.add_query(bool_query)
-            results = query.search(index=TERMS_INDEX)["hits"]["hits"]
-
-            for result in results:
-                se.delete(index=TERMS_INDEX, id=result["_id"])
+            if index:
+                query = Query(se)
+                bool_query = Bool()
+                bool_query.filter(Terms(field="tileid", terms=[self.tileid]))
+                query.add_query(bool_query)
+                results = query.delete(index=TERMS_INDEX)
 
             self.__preDelete(request)
             self.save_edit(
-                user=request.user, edit_type="tile delete", old_value=self.data, provisional_edit_log_details=provisional_edit_log_details
+                user=user,
+                edit_type="tile delete",
+                old_value=self.data,
+                provisional_edit_log_details=provisional_edit_log_details,
+                transaction_id=transaction_id,
             )
             try:
                 super(Tile, self).delete(*args, **kwargs)
-                for nodeid, value in self.data.items():
+                for nodeid in self.data.keys():
                     node = models.Node.objects.get(nodeid=nodeid)
                     datatype = self.datatype_factory.get_instance(node.datatype)
-                    datatype.post_tile_delete(self, nodeid)
-                resource = Resource.objects.get(resourceinstanceid=self.resourceinstance.resourceinstanceid)
-                resource.index()
-            except IntegrityError:
-                logger.error
+                    datatype.post_tile_delete(self, nodeid, index=index)
+                if index:
+                    self.index()
+            except IntegrityError as e:
+                logger.error(e)
 
         else:
             self.apply_provisional_edit(user, data={}, action="delete")
@@ -470,7 +534,7 @@ class Tile(models.TileModel):
         nodegroup = models.NodeGroup.objects.get(pk=self.nodegroup_id)
         for node in nodegroup.node_set.all():
             datatype = self.datatype_factory.get_instance(node.datatype)
-            datatype.after_update_all()
+            datatype.after_update_all(tile=self)
         for tile in self.tiles:
             tile.after_update_all()
 
@@ -489,18 +553,13 @@ class Tile(models.TileModel):
 
     @staticmethod
     def get_blank_tile(nodeid, resourceid=None):
-        parent_nodegroup = None
-
-        node = models.Node.objects.get(pk=nodeid)
-        if node.nodegroup.parentnodegroup_id is not None:
-            parent_nodegroup = node.nodegroup.parentnodegroup
-            parent_tile = Tile()
-            parent_tile.data = {}
+        node = models.Node.objects.filter(pk=nodeid).select_related("nodegroup")[0]
+        parentnodegroup_id = node.nodegroup.parentnodegroup_id
+        if parentnodegroup_id is not None:
+            parent_tile = Tile.get_blank_tile_from_nodegroup_id(nodegroup_id=parentnodegroup_id, resourceid=resourceid, parenttile=None)
             parent_tile.tileid = None
-            parent_tile.nodegroup_id = node.nodegroup.parentnodegroup_id
-            parent_tile.resourceinstance_id = resourceid
             parent_tile.tiles = []
-            for nodegroup in models.NodeGroup.objects.filter(parentnodegroup_id=node.nodegroup.parentnodegroup_id):
+            for nodegroup in models.NodeGroup.objects.filter(parentnodegroup_id=parentnodegroup_id):
                 parent_tile.tiles.append(Tile.get_blank_tile_from_nodegroup_id(nodegroup.pk, resourceid=resourceid, parenttile=parent_tile))
             return parent_tile
         else:
@@ -515,51 +574,71 @@ class Tile(models.TileModel):
         tile.data = {}
 
         for node in models.Node.objects.filter(nodegroup=nodegroup_id):
-            tile.data[str(node.nodeid)] = None
+            if node.datatype != "semantic":
+                tile.data[str(node.nodeid)] = None
 
         return tile
 
     @staticmethod
-    def update_node_value(nodeid, value, tileid=None, nodegroupid=None, resourceinstanceid=None):
+    def update_node_value(nodeid, value, tileid=None, nodegroupid=None, request=None, resourceinstanceid=None, transaction_id=None):
         """
         Updates the value of a node in a tile. Creates the tile and parent tiles if they do not yet
         exist.
 
         """
-
         if tileid and models.TileModel.objects.filter(pk=tileid).exists():
             tile = Tile.objects.get(pk=tileid)
             tile.data[nodeid] = value
-            tile.save()
+            tile.save(request=request, transaction_id=transaction_id)
         elif models.TileModel.objects.filter(Q(resourceinstance_id=resourceinstanceid), Q(nodegroup_id=nodegroupid)).count() == 1:
             tile = Tile.objects.filter(Q(resourceinstance_id=resourceinstanceid), Q(nodegroup_id=nodegroupid))[0]
             tile.data[nodeid] = value
-            tile.save()
+            tile.save(request=request, transaction_id=transaction_id)
         else:
+            new_resource_created = False
             if not resourceinstanceid:
                 graph = models.Node.objects.get(pk=nodeid).graph
                 resource_instance = models.ResourceInstance(graph=graph)
                 resource_instance.save()
                 resourceinstanceid = str(resource_instance.resourceinstanceid)
+                new_resource_created = True
             tile = Tile.get_blank_tile(nodeid, resourceinstanceid)
             if nodeid in tile.data:
                 tile.data[nodeid] = value
-                tile.save()
+                tile.save(request=request, new_resource_created=new_resource_created, transaction_id=transaction_id)
             else:
-                tile.save()
+                tile.save(request=request, new_resource_created=new_resource_created, transaction_id=transaction_id)
+                if not nodegroupid:
+                    nodegroupid = models.Node.objects.get(pk=nodeid).nodegroup_id
                 if nodegroupid and resourceinstanceid:
-                    tile = Tile.update_node_value(nodeid, value, nodegroupid=nodegroupid, resourceinstanceid=resourceinstanceid)
+                    tile = Tile.update_node_value(
+                        nodeid,
+                        value,
+                        nodegroupid=nodegroupid,
+                        request=request,
+                        resourceinstanceid=resourceinstanceid,
+                        transaction_id=transaction_id,
+                    )
+
+        tile.after_update_all()
         return tile
 
-    def __preSave(self, request=None):
+    def __preSave(self, request=None, context=None):
+        """
+        Keyword Arguments:
+        request -- request object passed from the view to the model.
+        context -- string e.g. "copy" indicating conditions under which a resource is saved and how functions should behave.
+        """
+
         try:
             for function in self._getFunctionClassInstances():
                 try:
-                    function.save(self, request)
+                    function.save(self, request, context=context)
                 except NotImplementedError:
                     pass
-        except TypeError:
-            logger.info(_("No associated functions or other TypeError raised by a function"))
+        except TypeError as e:
+            logger.warning(_("No associated functions or other TypeError raised by a function"))
+            logger.warning(e)
 
     def __preDelete(self, request):
         try:
@@ -568,14 +647,21 @@ class Tile(models.TileModel):
                     function.delete(self, request)
                 except NotImplementedError:
                     pass
-        except TypeError:
-            logger.info(_("No associated functions or other TypeError raised by a function"))
+        except TypeError as e:
+            logger.warning(_("No associated functions or other TypeError raised by a function"))
+            logger.warning(e)
+
+    def __postSave(self, request=None, context=None):
+        """
+        Keyword Arguments:
+        request -- request object passed from the view to the model.
+        context -- string e.g. "copy" indicating conditions under which a resource is saved and how functions should behave.
+        """
 
-    def __postSave(self, request=None):
         try:
             for function in self._getFunctionClassInstances():
                 try:
-                    function.post_save(self, request)
+                    function.post_save(self, request, context=context)
                 except NotImplementedError:
                     pass
         except TypeError as e:
@@ -588,7 +674,7 @@ class Tile(models.TileModel):
         functionXgraphs = models.FunctionXGraph.objects.filter(
             Q(graph_id=resource.graph_id),
             Q(config__contains={"triggering_nodegroups": [str(self.nodegroup_id)]}) | Q(config__triggering_nodegroups__exact=[]),
-            ~Q(function__classname="PrimaryDescriptorsFunction"),
+            ~Q(function__functiontype="primarydescriptors"),
         )
         for functionXgraph in functionXgraphs:
             func = functionXgraph.function.get_class_module()(functionXgraph.config, self.nodegroup_id)
@@ -623,3 +709,9 @@ class TileValidationError(Exception):
 
     def __str__(self):
         return repr(self.message)
+
+
+class TileCardinalityError(TileValidationError):
+    def __init__(self, message, code=None):
+        super(TileCardinalityError, self).__init__(message, code)
+        self.title = _("Tile Cardinaltiy Error")
