@@ -22,7 +22,9 @@ from arches.app.models.models import (
     ResourceInstance,
     FunctionXGraph,
     GraphXMapping,
+    TileModel,
 )
+from arches.app.models.card import Card
 from arches.app.utils.data_management.resource_graphs import exporter as GraphExporter
 from arches.app.models.resource import Resource
 from arches.app.models.system_settings import settings
@@ -43,45 +45,6 @@ class MissingConfigException(Exception):
     def __str__(self):
         return repr(self.value)
 
-
-class ConceptLookup:
-    def __init__(self, create=False):
-        self.lookups = {}
-        self.create = create
-        self.add_domain_values_to_lookups()
-
-    def lookup_label(self, label, collectionid):
-        ret = label
-        collection_values = self.lookups[collectionid]
-        for concept in collection_values:
-            if label == concept[1]:
-                ret = concept[2]
-        return ret
-
-    def lookup_labelid_from_label(self, value, collectionid):
-        ret = []
-        for val in csv.reader([value], delimiter=",", quotechar='"'):
-            for v in val:
-                v = v.strip()
-                try:
-                    ret.append(self.lookup_label(v, collectionid))
-                except:
-                    self.lookups[collectionid] = Concept().get_child_collections(collectionid)
-                    ret.append(self.lookup_label(v, collectionid))
-        output = StringIO()
-        writer = csv.writer(output)
-        writer.writerow(ret)
-        for v in [output.getvalue()]:
-            return v.strip("\r\n")
-
-    def add_domain_values_to_lookups(self):
-        for node in Node.objects.filter(Q(datatype="domain-value") | Q(datatype="domain-value-list")):
-            domain_collection_id = str(node.nodeid)
-            self.lookups[domain_collection_id] = []
-            for val in node.config["options"]:
-                self.lookups[domain_collection_id].append(("0", val["text"], val["id"]))
-
-
 class CsvWriter(Writer):
     def __init__(self, **kwargs):
         super(CsvWriter, self).__init__(**kwargs)
@@ -164,7 +127,12 @@ class CsvWriter(Writer):
                                 concept_export_value_type = None
                                 if k in concept_export_value_lookup:
                                     concept_export_value_type = concept_export_value_lookup[k]
-                                value = self.transform_value_for_export(self.node_datatypes[k], tile.data[k], concept_export_value_type, k,)
+                                value = self.transform_value_for_export(
+                                    self.node_datatypes[k],
+                                    tile.data[k],
+                                    concept_export_value_type,
+                                    k,
+                                )
                                 other_group_record[mapping[k]] = value
                         else:
                             del tile.data[k]
@@ -193,7 +161,10 @@ class CsvWriter(Writer):
             csvwriter = csv.DictWriter(dest, delimiter=",", fieldnames=csv_header)
             csvwriter.writeheader()
             csvs_for_export.append(
-                {"name": csv_name.split(".")[0] + "_groups." + csv_name.split(".")[1], "outputfile": dest,}
+                {
+                    "name": csv_name.split(".")[0] + "_groups." + csv_name.split(".")[1],
+                    "outputfile": dest,
+                }
             )
             for csv_record in other_group_records:
                 if "populated_node_groups" in csv_record:
@@ -227,6 +198,10 @@ class CsvWriter(Writer):
                 "resourceinstanceidfrom",
                 "resourceinstanceidto",
                 "relationshiptype",
+                "resourceinstancefrom_graphid",
+                "resourceinstanceto_graphid",
+                "nodeid",
+                "tileid",
                 "datestarted",
                 "dateended",
                 "notes",
@@ -251,73 +226,113 @@ class CsvWriter(Writer):
 class TileCsvWriter(Writer):
     def __init__(self, **kwargs):
         super(TileCsvWriter, self).__init__(**kwargs)
-        self.node_datatypes = {
-            str(nodeid): datatype
-            for nodeid, datatype in Node.objects.values_list("nodeid", "datatype").filter(~Q(datatype="semantic"), graph__isresource=True)
-        }
+        self.node_datatypes = {}
+        self.datatype_factory = DataTypeFactory()
+
+        nodes = Node.objects.all().values("nodeid", "name")
+        self.node_name_lookup = {}
+        for node in nodes:
+            self.node_name_lookup[str(node["nodeid"])] = node["name"]
+
+    def group_tiles(self, tiles, key):
+        new_tiles = {}
+
+        for tile in tiles:
+            new_tiles[str(tile[key])] = []
+
+        for tile in tiles:
+            if str(tile[key]) in new_tiles.keys():
+                new_tiles[str(tile[key])].append(tile)
+
+        return new_tiles
+
+    def lookup_node_name(self, nodeid):
+        try:
+            node_name = self.node_name_lookup[nodeid]
+        except KeyError:
+            node_name = nodeid
+
+        return node_name
+
+    def lookup_node_value(self, value, nodeid):
+        if nodeid in self.node_datatypes:
+            datatype = self.node_datatypes[nodeid]
+        else:
+            datatype = self.datatype_factory.get_instance(Node.objects.get(nodeid=nodeid).datatype)
+            self.node_datatypes[nodeid] = datatype
+
+        if value is not None:
+            value = self.node_datatypes[nodeid].transform_export_values(value, concept_export_value_type="label", node=nodeid)
 
-    def transform_value_for_export(self, datatype, value, concept_export_value_type, node):
-        datatype_instance = self.datatype_factory.get_instance(datatype)
-        value = datatype_instance.transform_export_values(value, concept_export_value_type=concept_export_value_type, node=node)
         return value
 
+    def flatten_tile(self, tile, semantic_nodes):
+        for nodeid in tile["data"]:
+            if nodeid not in semantic_nodes:
+                node_name = self.lookup_node_name(nodeid)
+                node_value = self.lookup_node_value(tile["data"][nodeid], nodeid)
+                tile[node_name] = node_value
+        del tile["data"]
+
+        return tile
+
+    def sort_field_names(self, columns=[]):
+        """
+        Moves sortorder, provisionaledits and nodegroup_id to last position and
+        ensures the first columns are ordered as tileid, parenttile_id, and resourceinstance_id.
+        """
+
+        columns_to_move_to_end = ["sortorder", "provisionaledits", "nodegroup_id"]
+        for name in columns_to_move_to_end:
+            columns.insert(len(columns) + 1, columns.pop(columns.index(name)))
+        columns_to_move_to_start = ["resourceinstance_id", "parenttile_id", "tileid"]
+        for name in columns_to_move_to_start:
+            columns.insert(0, columns.pop(columns.index(name)))
+
     def write_resources(self, graph_id=None, resourceinstanceids=None, **kwargs):
         super(TileCsvWriter, self).write_resources(graph_id=graph_id, resourceinstanceids=resourceinstanceids, **kwargs)
 
-        csv_records = []
-        other_group_records = []
-        concept_export_value_lookup = {}
-        csv_header = ["ResourceID"]
-        mapping = {}
-        nodes = Node.objects.filter(graph_id=self.graph_id)
-        for node in nodes:
-            mapping[str(node.nodeid)] = node.name
-        csv_header = ["ResourceID", "ResourceLegacyID", "ResourceModelID", "TileID", "ParentTileID", "NodeGroupID"] + list(mapping.values())
         csvs_for_export = []
 
-        for resourceinstanceid, tiles in self.resourceinstances.items():
-            tiles = sorted(tiles, key=lambda k: k.parenttile_id)
-            for tile in tiles:
-                csv_record = {}
-                csv_record["ResourceID"] = resourceinstanceid
-                csv_record["ResourceModelID"] = self.graph_id
-                csv_record["TileID"] = tile.tileid
-                csv_record["ParentTileID"] = str(tile.parenttile_id)
-                csv_record["NodeGroupID"] = str(tile.nodegroup_id)
-                for k in list(tile.data.keys()):
-                    resource_instance = ResourceInstance.objects.get(resourceinstanceid=resourceinstanceid)
-                    csv_record["ResourceLegacyID"] = (
-                        str(resource_instance.legacyid)
-                        if resource_instance.legacyid is not None
-                        else str(resource_instance.resourceinstanceid)
-                    )
-                    if tile.data[k] != "" and tile.data[k] is not None:
-                        concept_export_value_type = "label"
-                        if k in concept_export_value_lookup:
-                            concept_export_value_type = concept_export_value_lookup[k]
-                        if tile.data[k] is not None:
-                            value = self.transform_value_for_export(self.node_datatypes[k], tile.data[k], concept_export_value_type, k)
-                            csv_record[mapping[k]] = value
-                        del tile.data[k]
-                    else:
-                        del tile.data[k]
+        if graph_id:
+            tiles = self.group_tiles(
+                list(TileModel.objects.filter(resourceinstance__graph_id=graph_id).order_by("nodegroup_id").values()), "nodegroup_id"
+            )
+        else:
+            tiles = self.group_tiles(
+                list(TileModel.objects.filter(resourceinstance_id__in=resourceinstanceids).order_by("nodegroup_id").values()),
+                "nodegroup_id",
+            )
+        semantic_nodes = [str(n[0]) for n in Node.objects.filter(datatype="semantic").values_list("nodeid")]
 
-                if csv_record != {"ResourceID": resourceinstanceid}:
-                    csv_records.append(csv_record)
+        for nodegroupid, nodegroup_tiles in tiles.items():
+            flattened_tiles = []
+            fieldnames = []
+            for tile in nodegroup_tiles:
+                tile["tileid"] = str(tile["tileid"])
+                tile["parenttile_id"] = str(tile["parenttile_id"])
+                tile["resourceinstance_id"] = str(tile["resourceinstance_id"])
+                flattened_tile = self.flatten_tile(tile, semantic_nodes)
+                tile["nodegroup_id"] = str(tile["nodegroup_id"])
+                flattened_tiles.append(flattened_tile)
 
-        dest = StringIO()
-        csvwriter = csv.DictWriter(dest, delimiter=",", fieldnames=csv_header)
-        csvwriter.writeheader()
-        all_records = sorted(csv_records, key=lambda k: k["ResourceID"])
-        for csv_record in all_records:
-            if "populated_node_groups" in csv_record:
-                del csv_record["populated_node_groups"]
-            csvwriter.writerow({k: str(v) for k, v in list(csv_record.items())})
+                for fieldname in flattened_tile:
+                    if fieldname not in fieldnames:
+                        fieldnames.append(fieldname)
 
-        csv_name = os.path.join("{0}.{1}".format(self.file_name, "csv"))
-        csvs_for_export.append({"name": csv_name, "outputfile": dest})
-        if self.graph_id is not None:
-            csvs_for_export = csvs_for_export
+            self.sort_field_names(fieldnames)
+            tiles[nodegroupid] = sorted(flattened_tiles, key=lambda k: k["resourceinstance_id"])
+            dest = StringIO()
+            csvwriter = csv.DictWriter(dest, delimiter=",", fieldnames=fieldnames)
+            csvwriter.writeheader()
+            csv_name = os.path.join("{0}.{1}".format(Card.objects.get(nodegroup_id=nodegroupid).name, "csv"))
+            forbidden_excel_sheet_name_characters = ["\\", "/", "?", "*", "[", "]"]
+            for character in forbidden_excel_sheet_name_characters:
+                if character in csv_name:
+                    csv_name = csv_name.replace(character, "_")
+            for v in tiles[nodegroupid]:
+                csvwriter.writerow(v)
+            csvs_for_export.append({"name": csv_name, "outputfile": dest})
 
         return csvs_for_export
 
@@ -327,7 +342,19 @@ class CsvReader(Reader):
         self.errors = []
         super(CsvReader, self).__init__()
 
-    def save_resource(self, populated_tiles, resourceinstanceid, legacyid, resources, target_resource_model, bulk, save_count, row_number):
+    def save_resource(
+        self,
+        populated_tiles,
+        resourceinstanceid,
+        legacyid,
+        resources,
+        target_resource_model,
+        bulk,
+        save_count,
+        row_number,
+        prevent_indexing,
+        transaction_id=None,
+    ):
         # create a resource instance only if there are populated_tiles
         errors = []
         if len(populated_tiles) > 0:
@@ -343,11 +370,11 @@ class CsvReader(Reader):
             if bulk:
                 resources.append(newresourceinstance)
                 if len(resources) >= settings.BULK_IMPORT_BATCH_SIZE:
-                    Resource.bulk_save(resources=resources)
+                    Resource.bulk_save(resources=resources, transaction_id=transaction_id)
                     del resources[:]  # clear out the array
             else:
                 try:
-                    newresourceinstance.save()
+                    newresourceinstance.save(index=(not prevent_indexing), transaction_id=transaction_id)
 
                 except TransportError as e:
 
@@ -362,7 +389,7 @@ class CsvReader(Reader):
                     newresourceinstance.delete()
                     save_count = save_count - 1
                 except Exception as e:
-                    msg = "%s: WARNING: failed to index document in resource: %s %s. Exception detail:\n%s\n" % (
+                    msg = "%s: WARNING: failed to save document in resource: %s %s. Exception detail:\n%s\n" % (
                         datetime.datetime.now(),
                         resourceinstanceid,
                         row_number,
@@ -388,7 +415,15 @@ class CsvReader(Reader):
             print("%s resources processed" % str(save_count))
 
     def import_business_data(
-        self, business_data=None, mapping=None, overwrite="append", bulk=False, create_concepts=False, create_collections=False
+        self,
+        business_data=None,
+        mapping=None,
+        overwrite="append",
+        bulk=False,
+        create_concepts=False,
+        create_collections=False,
+        prevent_indexing=False,
+        transaction_id=None,
     ):
         # errors = businessDataValidator(self.business_data)
         celery_worker_running = task_management.check_if_celery_available()
@@ -398,21 +433,25 @@ class CsvReader(Reader):
 
         def get_display_nodes(graphid):
             display_nodeids = []
-            functions = FunctionXGraph.objects.filter(function_id="60000000-0000-0000-0000-000000000001", graph_id=graphid)
+            functions = FunctionXGraph.objects.filter(function__functiontype="primarydescriptors", graph_id=graphid)
             for function in functions:
                 f = function.config
                 del f["triggering_nodegroups"]
 
-                for k, v in f.items():
+                for k, v in f["descriptor_types"].items():
                     v["node_ids"] = []
-                    v["string_template"] = v["string_template"].replace("<", "").replace(">", "").split(", ")
+                    v["string_template"] = (
+                        v["string_template"].replace("<", "").replace(">", "").split(", ")
+                        if "string_template" in v
+                        else ""
+                    )
                     if "nodegroup_id" in v and v["nodegroup_id"] != "":
                         nodes = Node.objects.filter(nodegroup_id=v["nodegroup_id"])
                         for node in nodes:
                             if node.name in v["string_template"]:
                                 display_nodeids.append(str(node.nodeid))
 
-                for k, v in f.items():
+                for k, v in f["descriptor_types"].items():
                     if "string_template" in v and v["string_template"] != [""]:
                         print(
                             "The {0} {1} in the {2} display function.".format(
@@ -489,7 +528,7 @@ class CsvReader(Reader):
                 }
                 display_nodes = get_display_nodes(graphid)
                 all_nodes = Node.objects.filter(graph_id=graphid)
-                node_list = {str(node.pk): node for node in all_nodes}
+                node_dict = {str(node.pk): node for node in all_nodes}
                 datatype_factory = DataTypeFactory()
                 concepts_to_create = {}
                 new_concepts = {}
@@ -676,8 +715,6 @@ class CsvReader(Reader):
 
                 if create_concepts == True:
                     create_reference_data(concepts_to_create, create_collections)
-                # if concepts are created on import concept_lookup must be instatiated afterward
-                concept_lookup = ConceptLookup()
 
                 def cache(blank_tile):
                     if blank_tile.data != {}:
@@ -720,18 +757,13 @@ class CsvReader(Reader):
                     if datatype != "":
                         errors = []
                         datatype_instance = datatype_factory.get_instance(datatype)
-                        if datatype in ["concept", "domain-value", "concept-list", "domain-value-list"]:
-                            try:
-                                uuid.UUID(value)
-                            except:
-                                if datatype in ["domain-value", "domain-value-list"]:
-                                    collection_id = nodeid
-                                else:
-                                    collection_id = Node.objects.get(nodeid=nodeid).config["rdmCollection"]
-                                if collection_id is not None:
-                                    value = concept_lookup.lookup_labelid_from_label(value, collection_id)
                         try:
-                            value = datatype_instance.transform_value_for_tile(value)
+                            if datatype in ["concept", "concept-list"]:
+                                config = Node.objects.get(nodeid=nodeid).config
+                                config["nodeid"] = nodeid
+                            else:
+                                config = {}
+                            value = datatype_instance.transform_value_for_tile(value, **config)
                             errors = datatype_instance.validate(value, row_number=row_number, source=source, nodeid=nodeid)
                         except Exception as e:
                             errors.append(
@@ -755,11 +787,16 @@ class CsvReader(Reader):
 
                     return {"value": value, "request": request}
 
-                def get_blank_tile(source_data):
+                def get_blank_tile(source_data, child_only=False):
                     if len(source_data) > 0:
                         if source_data[0] != {}:
                             key = str(list(source_data[0].keys())[0])
-                            if key not in blanktilecache:
+                            source_node = node_dict[key]
+                            if child_only:
+                                blank_tile = Tile.get_blank_tile_from_nodegroup_id(str(source_node.nodegroup_id))
+                                blank_tile.tiles = []
+                                blank_tile.tileid = None
+                            elif key not in blanktilecache:
                                 blank_tile = Tile.get_blank_tile(key)
                                 cache(blank_tile)
                             else:
@@ -773,6 +810,8 @@ class CsvReader(Reader):
 
                 def check_required_nodes(tile, parent_tile, required_nodes, all_nodes):
                     # Check that each required node in a tile is populated.
+                    if settings.BYPASS_REQUIRED_VALUE_TILE_VALIDATION:
+                        return
                     errors = []
                     if len(required_nodes) > 0:
                         if bool(tile.data):
@@ -810,7 +849,16 @@ class CsvReader(Reader):
 
                         save_count = save_count + 1
                         self.save_resource(
-                            populated_tiles, resourceinstanceid, legacyid, resources, target_resource_model, bulk, save_count, row_number
+                            populated_tiles,
+                            resourceinstanceid,
+                            legacyid,
+                            resources,
+                            target_resource_model,
+                            bulk,
+                            save_count,
+                            row_number,
+                            prevent_indexing,
+                            transaction_id=transaction_id,
                         )
 
                         # reset values for next resource instance
@@ -853,7 +901,7 @@ class CsvReader(Reader):
                         if "NodeGroupID" in row and row["NodeGroupID"] is not None:
                             target_tile.nodegroupid = row["NodeGroupID"]
 
-                        def populate_tile(source_data, target_tile):
+                        def populate_tile(source_data, tile_to_populate, appending_to_parent=False):
                             """
                             source_data = [{nodeid:value},{nodeid:value},{nodeid:value} . . .]
                             All nodes in source_data belong to the same resource.
@@ -862,36 +910,39 @@ class CsvReader(Reader):
                             """
                             need_new_tile = False
                             # Set target tileid to None because this will be a new tile, a new tileid will be created on save.
-                            target_tile.tileid = uuid.uuid4()
+                            tile_to_populate.tileid = uuid.uuid4()
                             if "TileID" in row and row["TileID"] is not None:
-                                target_tile.tileid = row["TileID"]
-                            target_tile.resourceinstance_id = resourceinstanceid
+                                tile_to_populate.tileid = row["TileID"]
+                            tile_to_populate.resourceinstance_id = resourceinstanceid
                             # Check the cardinality of the tile and check if it has been populated.
                             # If cardinality is one and the tile is populated the tile should not be populated again.
-                            if str(target_tile.nodegroup_id) in single_cardinality_nodegroups and "TileiD" not in row:
+                            if str(tile_to_populate.nodegroup_id) in single_cardinality_nodegroups and "TileiD" not in row:
                                 target_tile_cardinality = "1"
                             else:
                                 target_tile_cardinality = "n"
 
-                            if str(target_tile.nodegroup_id) not in populated_nodegroups[resourceinstanceid]:
-                                target_tile.nodegroup_id = str(target_tile.nodegroup_id)
+                            if str(tile_to_populate.nodegroup_id) not in populated_nodegroups[resourceinstanceid] or appending_to_parent:
+                                tile_to_populate.nodegroup_id = str(tile_to_populate.nodegroup_id)
                                 # Check if we are populating a parent tile by inspecting the target_tile.data array.
-                                if target_tile.data != {}:
-                                    # Iterate through the target_tile nodes and begin populating by iterating througth source_data array.
-                                    # The idea is to populate as much of the target_tile as possible,
-                                    # before moving on to the next target_tile.
-                                    for target_key in list(target_tile.data.keys()):
+                                source_data_has_target_tile_nodes = (
+                                    len(set([list(obj.keys())[0] for obj in source_data]) & set(tile_to_populate.data.keys())) > 0
+                                )
+                                if source_data_has_target_tile_nodes:
+                                    # Iterate through tile_to_populate nodes and begin populating by iterating througth source_data array.
+                                    # The idea is to populate as much of the tile_to_populate as possible,
+                                    # before moving on to the next tile_to_populate.
+                                    for target_key in list(tile_to_populate.data.keys()):
                                         for source_tile in source_data:
                                             for source_key in list(source_tile.keys()):
                                                 # Check for source and target key match.
                                                 if source_key == target_key:
-                                                    if target_tile.data[source_key] is None:
+                                                    if tile_to_populate.data[source_key] is None:
                                                         # If match populate target_tile node with transformed value.
                                                         value = transform_value(
                                                             node_datatypes[source_key], source_tile[source_key], row_number, source_key
                                                         )
-                                                        target_tile.data[source_key] = value["value"]
-                                                        # target_tile.request = value['request']
+                                                        tile_to_populate.data[source_key] = value["value"]
+                                                        # tile_to_populate.request = value['request']
                                                         # Delete key from source_tile so
                                                         # we do not populate another tile based on the same data.
                                                         del source_tile[source_key]
@@ -899,10 +950,10 @@ class CsvReader(Reader):
                                     source_data[:] = [item for item in source_data if item != {}]
 
                                 # Check if we are populating a child tile(s) by inspecting the target_tiles.tiles array.
-                                elif target_tile.tiles is not None:
+                                elif tile_to_populate.tiles is not None:
                                     populated_child_tiles = []
                                     populated_child_nodegroups = []
-                                    for childtile in target_tile.tiles:
+                                    for childtile in tile_to_populate.tiles:
                                         if str(childtile.nodegroup_id) in single_cardinality_nodegroups:
                                             child_tile_cardinality = "1"
                                         else:
@@ -912,7 +963,7 @@ class CsvReader(Reader):
                                             prototype_tile_copy = pickle.loads(pickle.dumps(childtile, -1))
                                             tileid = row["TileID"] if "TileID" in row else uuid.uuid4()
                                             prototype_tile_copy.tileid = tileid
-                                            prototype_tile_copy.parenttile = target_tile
+                                            prototype_tile_copy.parenttile = tile_to_populate
                                             parenttileid = (
                                                 row["ParentTileID"] if "ParentTileID" in row and row["ParentTileID"] is not None else None
                                             )
@@ -935,7 +986,7 @@ class CsvReader(Reader):
                                                                     prototype_tile_copy.data[source_key] = value["value"]
                                                                     # print(prototype_tile_copy.data[source_key]
                                                                     # print('&'*80
-                                                                    # target_tile.request = value['request']
+                                                                    # tile_to_populate.request = value['request']
                                                                     del source_column[source_key]
                                                                 else:
                                                                     populate_child_tiles(source_data)
@@ -953,16 +1004,16 @@ class CsvReader(Reader):
 
                                         populate_child_tiles(source_data)
 
-                                    target_tile.tiles = populated_child_tiles
+                                    tile_to_populate.tiles = populated_child_tiles
 
-                                if not target_tile.is_blank():
-                                    populated_tiles.append(target_tile)
+                                if not tile_to_populate.is_blank() and not appending_to_parent:
+                                    populated_tiles.append(tile_to_populate)
 
                                 if len(source_data) > 0:
                                     need_new_tile = True
 
                                 if target_tile_cardinality == "1" and "NodeGroupID" not in row:
-                                    populated_nodegroups[resourceinstanceid].append(str(target_tile.nodegroup_id))
+                                    populated_nodegroups[resourceinstanceid].append(str(tile_to_populate.nodegroup_id))
 
                                 if need_new_tile:
                                     new_tile = get_blank_tile(source_data)
@@ -971,7 +1022,30 @@ class CsvReader(Reader):
 
                         # mock_request_object = HttpRequest()
 
-                        if target_tile is not None and len(source_data) > 0:
+                        # identify whether a tile for this nodegroup on this resource already exists
+                        preexisting_tile_for_nodegroup = list(
+                            filter(
+                                lambda t: str(t.resourceinstance_id) == str(row["ResourceID"])
+                                and str(t.nodegroup_id) == str(target_tile.nodegroup_id),
+                                populated_tiles,
+                            )
+                        )
+                        if len(preexisting_tile_for_nodegroup) > 0:
+                            preexisting_tile_for_nodegroup = preexisting_tile_for_nodegroup[0]
+                        else:
+                            preexisting_tile_for_nodegroup = False
+
+                        # aggregates a tile of the nodegroup associated with source_data (via get_blank_tile)
+                        # onto the pre-existing tile who would be its parent
+                        if target_tile.nodegroup.cardinality == "1" and preexisting_tile_for_nodegroup and len(source_data) > 0:
+                            target_tile = get_blank_tile(source_data, child_only=True)
+                            populate_tile(source_data, target_tile, appending_to_parent=True)
+                            target_tile.parenttile = preexisting_tile_for_nodegroup
+                            preexisting_tile_for_nodegroup.tiles.append(target_tile)
+
+                        # populates a tile from parent-level nodegroup because
+                        # parent cardinality is N or because none exists yet on resource
+                        elif target_tile is not None and len(source_data) > 0:
                             populate_tile(source_data, target_tile)
                             # Check that required nodes are populated. If not remove tile from populated_tiles array.
                             check_required_nodes(target_tile, target_tile, required_nodes, all_nodes)
@@ -985,7 +1059,7 @@ class CsvReader(Reader):
                     if len(v) > 0:
                         errors.append(
                             {
-                                "type": "WARNING",
+                                "type": "INFO",
                                 "message": "{0} is null or not mapped on rows {1} and \
                                 participates in a display value function.".format(
                                     k, ",".join(v)
@@ -997,13 +1071,23 @@ class CsvReader(Reader):
 
                 if "legacyid" in locals():
                     self.save_resource(
-                        populated_tiles, resourceinstanceid, legacyid, resources, target_resource_model, bulk, save_count, row_number
+                        populated_tiles,
+                        resourceinstanceid,
+                        legacyid,
+                        resources,
+                        target_resource_model,
+                        bulk,
+                        save_count,
+                        row_number,
+                        prevent_indexing,
+                        transaction_id=transaction_id,
                     )
 
                 if bulk:
                     print("Time to create resource and tile objects: %s" % datetime.timedelta(seconds=time() - self.start))
-                    Resource.bulk_save(resources=resources)
-                print(_(f"Total resources saved: {save_count + 1}"))
+                    Resource.bulk_save(resources=resources, transaction_id=transaction_id)
+                save_count = save_count + 1
+                print(_("Total resources saved: {save_count}").format(**locals()))
 
         except Exception as e:
             exc_type, exc_value, exc_traceback = sys.exc_info()
