@@ -39,7 +39,8 @@ def use_cache(url):
         return True
 
 
-def load_document_and_cache(url):
+# PyLD 2.0 / JSON-LD 1.1 passes two params, we don't need the second
+def load_document_and_cache(url, cache=None):
     if url in docCache and use_cache(url):
         return docCache[url]
 
@@ -137,6 +138,8 @@ class RdfWriter(Writer):
                 pkg["range_tile_data"] = tile.data[str(edge.rangenode_id)]
             if str(edge.domainnode_id) in tile.data:
                 pkg["domain_tile_data"] = tile.data[str(edge.domainnode_id)]
+            elif tile.parenttile is not None and str(edge.domainnode_id) in tile.parenttile.data:
+                pkg["domain_tile_data"] = tile.parenttile.data[str(edge.domainnode_id)]
 
             rng_dt = self.datatype_factory.get_instance(pkg["r_datatype"])
             pkg["d_uri"] = dom_dt.get_rdf_uri(domainnode, pkg["domain_tile_data"], "d")
@@ -271,14 +274,16 @@ class JsonLdWriter(RdfWriter):
 
 
 class JsonLdReader(Reader):
-    def __init__(self):
-        super(JsonLdReader, self).__init__()
+    def __init__(self, *args, **kwargs):
+        super(JsonLdReader, self).__init__(*args, **kwargs)
         self.tiles = {}
         self.resources = []
         self.resource = None
         self.use_ids = False
         self.root_ontologyclass_lookup = {}
         self.graphtree = None
+        self.print_buf = []
+        self.verbosity = kwargs.get("verbosity", 1)
         self.logger = logging.getLogger(__name__)
         for graph in models.GraphModel.objects.filter(isresource=True):
             node = models.Node.objects.get(graph_id=graph.pk, istopnode=True)
@@ -362,7 +367,7 @@ class JsonLdReader(Reader):
                 self.logger.debug("Valid resourceid not found within `{0}`".format(value))
         return None
 
-    def read_resource(self, data, use_ids=False, resourceid=None, graphid=None):
+    def read_resource(self, data, use_ids=False, resourceid=None, graphid=None, expand_data=True):
         if graphid is None and self.graphtree is None:
             raise Exception("No graphid supplied to read_resource")
         elif self.graphtree is None:
@@ -370,6 +375,7 @@ class JsonLdReader(Reader):
 
         # Ensure we've reset from any previous call
         self.errors = {}
+        self.idcache = {}
         self.resources = []
         self.resource = None
         self.use_ids = use_ids
@@ -386,7 +392,8 @@ class JsonLdReader(Reader):
             self.shouldSortTiles = False
 
         for jsonld_document in data:
-            jsonld_document = expand(jsonld_document)[0]
+            if expand_data:  # this should always be true, we set this to false just for some unit tests
+                jsonld_document = expand(jsonld_document)[0]
 
             # Possibly bail very early
             if jsonld_document["@type"][0] != self.graphtree["class"]:
@@ -413,7 +420,15 @@ class JsonLdReader(Reader):
                 result = {"data": [jsonld_document["@id"]]}
             else:
                 result = {"data": [None]}
-            self.data_walk(jsonld_document, self.graphtree, result)
+            self.root_json_document = jsonld_document
+            try:
+                self.data_walk(jsonld_document, self.graphtree, result)
+            except:
+                if self.verbosity > 1:
+                    for line in self.print_buf:
+                        # print(line) # uncomment this line to print errors directly to the screen
+                        self.logger.debug(line)
+
 
     def is_semantic_node(self, graph_node):
         return self.datatype_factory.datatypes[graph_node["datatype_type"]].defaultwidget is None
@@ -426,9 +441,53 @@ class JsonLdReader(Reader):
                 return True
         return False
 
-    def data_walk(self, data_node, tree_node, result, tile=None):
+    def build_reference_cache(self, jsonld_document):
+        if "@id" in jsonld_document and "@type" in jsonld_document:
+            dataType = jsonld_document["@type"][0] if isinstance(jsonld_document["@type"], list) else jsonld_document["@type"]
+            self.idcache[jsonld_document["@id"]] = dataType
+        for key, value in jsonld_document.items():
+            if key in ["@id", "@type"]:
+                continue
+            if isinstance(value, list):
+                for item in value:
+                    self.build_reference_cache(item)
+            elif isinstance(value, dict):
+                self.build_reference_cache(value)
+
+    def get_cached_reference(self, lookup):
+        try:
+            return self.idcache[lookup]
+        except:
+            self.build_reference_cache(self.root_json_document)
+            try:
+                return self.idcache[lookup]
+            except:
+                raise ("Local reference not found")
+
+    def printline(self, text, indent=0, newline=False):
+        prefix = ""
+        if newline:
+            prefix = "\n"
+        if indent and indent > 0:
+            prefix = prefix + "   " * int(indent)
+        self.print_buf.append(prefix + text)
+
+    def data_walk(self, data_node, tree_node, result, tile=None, indent=0):
         my_tiles = []
+        self.printline(f"---" * 20, indent)
+        self.printline(tree_node["name"], indent)
+        self.printline(f"tile={tile}", indent)
+
+        # pre-seed as much of the cache as we can during the data-walk
+        if "@id" in data_node and "@type" in data_node:
+            dataType = data_node["@type"][0] if isinstance(data_node["@type"], list) else data_node["@type"]
+            self.idcache[data_node["@id"]] = dataType
         for k, v in data_node.items():
+            # self.printline(f"k: {k}", indent + 1)
+            # self.printline(f"v: {v}", indent + 1)
+            # k is a ontology property like
+            # "http://www.cidoc-crm.org/cidoc-crm/P1_is_identified_by"
+            # or "http://www.w3.org/2000/01/rdf-schema#label"
             if k in ["@id", "@type"]:
                 continue
             # always a list
@@ -446,8 +505,8 @@ class JsonLdReader(Reader):
                         clss = vi["@type"][0]
                     except:
                         # {"@id": "http://something/.../"}
-                        # with no @type. This is typically an external concept URI
-                        # Look for it in the children of current node
+                        # with no @type. This is typically an external concept URI reference to a resource instance
+                        # Look for it in the children of current node or in the entire document itself (if it's a resource instance ref)
                         possible_cls = []
                         for tn in tree_node["children"]:
                             if tn.startswith(k):
@@ -455,11 +514,20 @@ class JsonLdReader(Reader):
                         if len(possible_cls) == 1:
                             clss = possible_cls[0]
                         else:
-                            raise ValueError(f"Multiple possible branches and no @type given: {vi}")
+                            try:
+                                # this may be a reference to an entity already defined elsewhere in the json document
+                                # this can happen when there are more than 1 reference to the same resource instance
+                                clss = self.get_cached_reference(uri)
+                                vi["@type"] = clss
+                            except:
+                                raise ValueError(f"Multiple possible branches and no @type given: {vi}")
 
                     value = None
                     is_literal = False
 
+                # Here we try and find a possible match between the node_tree and data_tree
+                # we're matching "key" which equals the concatentaion of property and class
+                # at the same level in the trees
                 # Find precomputed possible branches by prop/class combination
                 key = f"{k} {clss}"
                 if key in tree_node["datatype"].ignore_keys():
@@ -474,32 +542,56 @@ class JsonLdReader(Reader):
                 elif not key in tree_node["children"]:
                     raise ValueError(f"property/class combination does not exist in model: {k} {clss}\nWhile processing: {vi}")
 
+                # if we made it this far then it means that we've found at least 1 match
                 options = tree_node["children"][key]
                 possible = []
                 ignore = []
 
+                # options is a list of potential matches in the graph tree
+                # based on property/class combination
+
                 for o in options:
-                    # print(f"Considering:\n  {vi}\n  {o['name']}")
+                    self.printline(f"Considering match to node: '{o['name']}'", indent + 1, newline=True)
+                    self.printline(f"Incoming value we're testing:  '{value or uri}'", indent + 1)
+                    self.printline(f"New Nodegroup = {o['nodegroup_id'] == o['node_id']}", indent + 1)
+                    self.printline(f"Parent tile id = {result['tile'].tileid if 'tile' in result else None}", indent + 1)
+                    potential_tile = None
+                    if o["node_id"] == o["nodegroup_id"]:
+                        # self.printline("--- getting potential tile  ---?", indent + 1)
+                        # Used to pick the previous tile in loop which MIGHT be the parent (but might not)
+                        parenttile_id = result["tile"].tileid if "tile" in result else None
+                        potential_tile = Tile(
+                            tileid=uuid.uuid4(),
+                            resourceinstance_id=self.resource.pk,
+                            parenttile_id=parenttile_id,
+                            nodegroup_id=o["nodegroup_id"],
+                            data={},
+                        )
                     if is_literal and o["datatype"].is_a_literal_in_rdf():
-                        if len(o["datatype"].validate(value)) == 0:
-                            possible.append([o, value])
+                        if len(o["datatype"].validate_from_rdf(value)) == 0:
+                            possible.append([o, value, potential_tile])
                         else:
-                            print(f"Could not validate {value} as a {o['datatype']}")
+                            self.printline(f"Could not validate {value} as a {o['datatype']}", indent + 1)
                     elif not is_literal and not o["datatype"].is_a_literal_in_rdf():
                         if self.is_concept_node(uri):
+                            self.printline("This is a concept node, so we'll test if the incoming data can fit here", indent + 1)
                             collid = o["config"]["collection_id"]
                             try:
                                 if self.validate_concept_in_collection(uri, collid):
-                                    possible.append([o, uri])
+                                    possible.append([o, uri, potential_tile])
+                                    self.printline(f"POSSIBLE match found.", indent + 1)
                                 else:
-                                    print(f"Concept URI {uri} not in Collection {collid}")
+                                    self.printline(
+                                        f"Match REJECTED!! Incoming concept URI {uri} not found in the nodes Collection {collid}",
+                                        indent + 1,
+                                    )
                             except:
-                                print(f"Errored testing concept {uri} in collection {collid}")
+                                self.printline(f"Errored testing concept {uri} in collection {collid}", indent + 1)
                         elif self.is_semantic_node(o):
-                            possible.append([o, ""])
+                            possible.append([o, "", potential_tile])
                         elif o["datatype"].accepts_rdf_uri(uri):
-                            # print(f"datatype for {o['name']} accepts uri")
-                            possible.append([o, uri])
+                            # self.printline(f"datatype for {o['name']} accepts uri", indent+1)
+                            possible.append([o, uri, potential_tile])
                         else:
                             # This is when the current option doesn't match, but could be
                             # non-ambiguous resource-instance vs semantic node
@@ -507,20 +599,24 @@ class JsonLdReader(Reader):
                     else:
                         raise ValueError("No possible match?")
 
-                # print(f"Possible is: {[x[0]['name'] for x in possible]}")
+                # self.printline(f"Possible is: {[x[0]['name'] for x in possible]}", indent+1)
 
                 if not possible:
-                    # print(f"Tried: {options}")
+                    # self.printline(f"Tried: {options}")
                     raise ValueError(f"Data does not match any actual node, despite prop/class combination {k} {clss}:\n{vi}")
                 elif len(possible) > 1:
                     # descend into data to check if there are further clarifying features
                     possible2 = []
                     for p in possible:
+                        # self.printline(f"\n---SECOND TIER: {p[0]['name']}", indent + 1)
                         try:
-                            # Don't really create data, so pass anonymous result dict
-                            self.data_walk(vi, p[0], {}, tile)
+                            # self.printline("Don't really create data, so pass anonymous result dict", indent + 1)
+                            if p[2] is not None:
+                                tile = p[2]
+                            self.data_walk(vi, p[0], {}, tile, indent + 1)
                             possible2.append(p)
-                        except:
+                        except Exception as e:
+                            self.printline(f"Failed due to {e}", indent + 1)
                             # Not an option
                             pass
                     if not possible2:
@@ -564,6 +660,8 @@ class JsonLdReader(Reader):
                     # Might get checked in a cardinality n branch that shouldn't be repeated
                     node_value = None
 
+                self.printline(f"A matching branch has been found and the value can be saved.", indent + 1)
+
                 # We know now that it can go into the branch
                 # Determine if we can collapse the data into a -list or not
                 bnodeid = branch[0]["node_id"]
@@ -619,7 +717,6 @@ class JsonLdReader(Reader):
                     else:
                         bnode["data"].append(branch[1])
                         if not self.is_semantic_node(branch[0]):
-                            # print(f"Adding to existing (n): {node_value}")
                             tile.data[bnodeid] = node_value
                         result[bnodeid].append(bnode)
                 else:
@@ -628,9 +725,10 @@ class JsonLdReader(Reader):
                     bnode["data"].append(branch[1])
                     result[bnodeid] = [bnode]
 
+                self.printline(f"Tile.data = {tile.data}", indent + 1)
                 if not is_literal:
-                    # Walk down non-literal branches in the data
-                    self.data_walk(vi, branch[0], bnode, tile)
+                    self.printline("Walk down non-literal branches in the data", indent + 1, newline=True)
+                    self.data_walk(vi, branch[0], bnode, tile, indent + 1)
 
         if self.shouldSortTiles:
             sortfuncs = settings.JSON_LD_SORT_FUNCTIONS
