@@ -17,13 +17,14 @@ along with this program. If not, see <http://www.gnu.org/licenses/>.
 """
 
 import uuid
-from django.db import transaction
+from django.db import transaction, connection
 from django.db.models import Q
 from django.http import HttpResponse, HttpResponseNotFound, HttpResponseNotAllowed, HttpResponseServerError
 from django.shortcuts import render
 from django.utils.decorators import method_decorator
 from django.utils.module_loading import import_string
 from django.utils.translation import ugettext as _
+from django.utils.translation import get_language
 from arches.app.models import models
 from arches.app.models.system_settings import settings
 from arches.app.models.concept import Concept, ConceptValue, CORE_CONCEPTS, get_preflabel_from_valueid
@@ -151,6 +152,10 @@ def concept(request, conceptid):
         for value in concept_graph.values:
             if value.category == "label":
                 labels.append(value)
+            if value.type == "image":
+                value.full_image_url = (
+                    (settings.FORCE_SCRIPT_NAME if settings.FORCE_SCRIPT_NAME is not None else "") + settings.MEDIA_URL + value.value
+                ).replace("//", "/")
 
         if (mode == "semantic" or mode == "") and (
             concept_graph.nodetype == "Concept" or concept_graph.nodetype == "ConceptScheme" or concept_graph.nodetype == "EntityType"
@@ -168,6 +173,7 @@ def concept(request, conceptid):
                 request,
                 "views/rdm/concept-report.htm",
                 {
+                    "FORCE_SCRIPT_NAME": settings.FORCE_SCRIPT_NAME,
                     "lang": lang,
                     "prefLabel": prefLabel,
                     "labels": labels,
@@ -386,6 +392,44 @@ def paged_dropdown(request):
         dict(list(zip(["id", "text", "conceptid", "language", "type"], d["valueto"].values())), depth=d["depth"], collector=d["collector"])
         for d in data
     ]
+
+    # This try/except block trys to find an exact match to the concept the user is searching and if found
+    # it will insert it into the results as the first item so that users don't have to scroll to find it.
+    # See: https://github.com/archesproject/arches/issues/8355
+    try:
+        if page == 1:
+            found = False
+            for i, d in enumerate(data):
+                if i <= 7 and d["text"].lower() == query.lower():
+                    found = True
+                    break
+            if not found:
+                sql = """
+                    SELECT value, valueid
+                    FROM 
+                    (
+                        SELECT *, CASE WHEN LOWER(languageid) = '{languageid}' THEN 10
+                        WHEN LOWER(languageid) like '{short_languageid}%' THEN 5
+                        ELSE 0
+                        END score
+                        FROM values
+                    ) as vals
+                    WHERE LOWER(value)='{query}' AND score > 0
+                    AND valuetype in ('prefLabel')
+                    ORDER BY score desc limit 1
+                """
+
+                languageid = get_language().lower()
+                sql = sql.format(query=query.lower(), languageid=languageid, short_languageid=languageid.split("-")[0])
+                cursor = connection.cursor()
+                cursor.execute(sql)
+                rows = cursor.fetchall()
+
+                if len(rows) == 1:
+                    data.insert(0, {"id": str(rows[0][1]), "text": rows[0][0], "depth": 1, "collector": False})
+    except:
+        pass
+
     return JSONResponse({"results": data, "more": offset + limit < total_count})
 
 
