@@ -160,10 +160,10 @@ class GraphDesignerView(GraphBaseView):
             except AttributeError as e:
                 logger.info(
                     _(
-                        f"No namespaces appear to be associated with {ontology.ontologyid} in the ontologies table."
+                        "No namespaces appear to be associated with {ontology.ontologyid} in the ontologies table."
                         " This is not a problem as long as all necessary namespaces are included in the"
                         " ONTOLOGY_NAMESPACES setting."
-                    )
+                    ).format(**locals())
                 )
         return ontology_namespaces
 
@@ -174,7 +174,16 @@ class GraphDesignerView(GraphBaseView):
         datatypes = models.DDataType.objects.all()
         datatypes_json = JSONSerializer().serialize(datatypes, exclude=["modulename", "isgeometric"])
         branch_graphs = Graph.objects.exclude(pk=graphid).exclude(isresource=True)
+
+        primary_descriptor_functions = models.FunctionXGraph.objects.filter(
+            graph=self.graph
+        ).filter(function__functiontype="primarydescriptors")
+        primary_descriptor_function = JSONSerializer().serialize(
+            primary_descriptor_functions[0] if len(primary_descriptor_functions) > 0 else None
+        )
+
         applied_functions = JSONSerializer().serialize(models.FunctionXGraph.objects.filter(graph=self.graph))
+
         cards = self.graph.cardmodel_set.order_by("sortorder").prefetch_related("cardxnodexwidget_set")
         constraints = []
         for card in cards:
@@ -222,6 +231,7 @@ class GraphDesignerView(GraphBaseView):
             map_markers=map_markers,
             map_sources=map_sources,
             applied_functions=applied_functions,
+            primary_descriptor_function=primary_descriptor_function,
             geocoding_providers=geocoding_providers,
             report_templates=templates,
             restricted_nodegroups=[str(nodegroup) for nodegroup in restricted_nodegroups],
@@ -400,7 +410,7 @@ class GraphDataView(View):
             return JSONErrorResponse(
                 _("Elasticsearch indexing error"),
                 _(
-                    """If you want to change the datatype of an existing node.  
+                    """If you want to change the datatype of an existing node.
                     Delete and then re-create the node, or export the branch then edit the datatype and re-import the branch."""
                 ),
             )
@@ -470,7 +480,7 @@ class CardView(GraphBaseView):
 
 
 class DatatypeTemplateView(TemplateView):
-    def get(sefl, request, template="text"):
+    def get(self, request, template="text"):
         return render(request, "views/components/datatypes/%s.htm" % template)
 
 
