@@ -16,11 +16,15 @@ You should have received a copy of the GNU Affero General Public License
 along with this program. If not, see <http://www.gnu.org/licenses/>.
 """
 
-import os
-import logging
+from base64 import b64decode
 from datetime import datetime
+import logging
+import os
+import json
+from django.contrib.auth import authenticate
 from django.contrib.gis.geos import GEOSGeometry
 from django.core.cache import cache
+from django.db import connection
 from django.http import HttpResponseNotFound
 from django.shortcuts import render
 from django.utils.translation import ugettext as _
@@ -41,11 +45,16 @@ from arches.app.models.concept import get_preflabel_from_conceptid
 from arches.app.utils.permission_backend import get_nodegroups_by_perm, user_is_resource_reviewer
 import arches.app.utils.zip as zip_utils
 import arches.app.utils.task_management as task_management
+from arches.app.utils.data_management.resources.formats.htmlfile import HtmlWriter
 import arches.app.tasks as tasks
 from io import StringIO
+from tempfile import NamedTemporaryFile
+from openpyxl import Workbook
+from arches.app.models.system_settings import settings
 
 logger = logging.getLogger(__name__)
 
+
 class SearchView(MapBaseManagerView):
     def get(self, request):
         map_layers = models.MapLayer.objects.all()
@@ -101,12 +110,19 @@ class SearchView(MapBaseManagerView):
             "template": "search-help",
         }
         context["celery_running"] = task_management.check_if_celery_available()
+        context["export_html_templates"] = HtmlWriter.get_graphids_with_export_template()
 
         return render(request, "views/search.htm", context)
 
 
 def home_page(request):
-    return render(request, "views/search.htm", {"main_script": "views/search",})
+    return render(
+        request,
+        "views/search.htm",
+        {
+            "main_script": "views/search",
+        },
+    )
 
 
 def search_terms(request):
@@ -195,26 +211,56 @@ def export_results(request):
 
     total = int(request.GET.get("total", 0))
     format = request.GET.get("format", "tilecsv")
-    download_limit = settings.SEARCH_EXPORT_IMMEDIATE_DOWNLOAD_THRESHOLD
-    if total > download_limit:
-        celery_worker_running = task_management.check_if_celery_available()
-        if celery_worker_running is True:
-            request_values = dict(request.GET)
-            request_values["path"] = request.get_full_path()
-            result = tasks.export_search_results.apply_async(
-                (request.user.id, request_values, format), link=tasks.update_user_task_record.s(), link_error=tasks.log_error.s()
-            )
+    report_link = request.GET.get("reportlink", False)
+    app_name = settings.APP_NAME
+    if format == "html":
+        download_limit = settings.SEARCH_EXPORT_IMMEDIATE_DOWNLOAD_THRESHOLD_HTML_FORMAT
+    else:
+        download_limit = settings.SEARCH_EXPORT_IMMEDIATE_DOWNLOAD_THRESHOLD
+
+    if total > download_limit and format != "geojson":
+        if (settings.RESTRICT_CELERY_EXPORT_FOR_ANONYMOUS_USER is True) and (request.user.username == "anonymous"):
             message = _(
-                f"{total} instances have been submitted for export. \
-                Click the Bell icon to check for a link to download your data"
-            )
-            return JSONResponse({"success": True, "message": message})
-        else:
-            message = _(f"Your search exceeds the {download_limit} instance download limit. Please refine your search")
+                "Your search exceeds the {download_limit} instance download limit.  \
+                Anonymous users cannot run an export exceeding this limit.  \
+                Please sign in with your {app_name} account or refine your search"
+            ).format(**locals())
             return JSONResponse({"success": False, "message": message})
+        else:
+            celery_worker_running = task_management.check_if_celery_available()
+            if celery_worker_running is True:
+                request_values = dict(request.GET)
+                request_values["path"] = request.get_full_path()
+                result = tasks.export_search_results.apply_async(
+                    (request.user.id, request_values, format, report_link),
+                    link=tasks.update_user_task_record.s(),
+                    link_error=tasks.log_error.s(),
+                )
+                message = _(
+                    "{total} instances have been submitted for export. \
+                    Click the Bell icon to check for a link to download your data"
+                ).format(**locals())
+                return JSONResponse({"success": True, "message": message})
+            else:
+                message = _("Your search exceeds the {download_limit} instance download limit. Please refine your search").format(
+                    **locals()
+                )
+                return JSONResponse({"success": False, "message": message})
+
+    elif format == "tilexl":
+        exporter = SearchResultsExporter(search_request=request)
+        export_files, export_info = exporter.export(format, report_link)
+        wb = export_files[0]["outputfile"]
+        with NamedTemporaryFile() as tmp:
+            wb.save(tmp.name)
+            tmp.seek(0)
+            stream = tmp.read()
+            export_files[0]["outputfile"] = tmp
+            return zip_utils.zip_response(export_files, zip_file_name=f"{settings.APP_NAME}_export.zip")
     else:
         exporter = SearchResultsExporter(search_request=request)
-        export_files, export_info = exporter.export(format)
+        export_files, export_info = exporter.export(format, report_link)
+
         if len(export_files) == 0 and format == "shp":
             message = _(
                 "Either no instances were identified for export or no resources have exportable geometry nodes\
@@ -236,20 +282,30 @@ def append_instance_permission_filter_dsl(request, search_results_object):
         search_results_object["query"].add_query(has_access)
 
 
-def search_results(request):
+def get_dsl_from_search_string(request):
+    dsl = search_results(request, returnDsl=True).dsl
+    return JSONResponse(dsl)
+
+
+def search_results(request, returnDsl=False):
     for_export = request.GET.get("export")
     pages = request.GET.get("pages", None)
     total = int(request.GET.get("total", "0"))
     resourceinstanceid = request.GET.get("id", None)
+    load_tiles = request.GET.get("tiles", False)
+    if load_tiles:
+        try:
+            load_tiles = json.loads(load_tiles)
+        except TypeError:
+            pass
     se = SearchEngineFactory().create()
+    permitted_nodegroups = get_permitted_nodegroups(request.user)
+    include_provisional = get_provisional_type(request)
+    search_filter_factory = SearchFilterFactory(request)
     search_results_object = {"query": Query(se)}
 
-    include_provisional = get_provisional_type(request)
-    permitted_nodegroups = get_permitted_nodegroups(request.user)
-
-    search_filter_factory = SearchFilterFactory(request)
     try:
-        for filter_type, querystring in list(request.GET.items()) + [("search-results", "")]:
+        for filter_type, querystring in list(request.GET.items()) + list(request.POST.items()) + [("search-results", "")]:
             search_filter = search_filter_factory.get_filter(filter_type)
             if search_filter:
                 search_filter.append_dsl(search_results_object, permitted_nodegroups, include_provisional)
@@ -259,6 +315,8 @@ def search_results(request):
         return JSONErrorResponse(message=err)
 
     dsl = search_results_object.pop("query", None)
+    if returnDsl:
+        return dsl
     dsl.include("graph_id")
     dsl.include("root_ontology_class")
     dsl.include("resourceinstanceid")
@@ -272,7 +330,7 @@ def search_results(request):
     dsl.include("displaydescription")
     dsl.include("map_popup")
     dsl.include("provisional_resource")
-    if request.GET.get("tiles", None) is not None:
+    if load_tiles:
         dsl.include("tiles")
     if for_export or pages:
         results = dsl.search(index=RESOURCES_INDEX, scroll="1m")
@@ -376,11 +434,16 @@ def _buffer(geojson, width=0, unit="ft"):
     if width > 0:
         if unit == "ft":
             width = width / 3.28084
-
-        geom.transform(settings.ANALYSIS_COORDINATE_SYSTEM_SRID)
-        geom = geom.buffer(width)
-        geom.transform(4326)
-
+        with connection.cursor() as cursor:
+            # Transform geom to the analysis SRID, buffer it, and transform it back to wgs84
+            cursor.execute(
+                """SELECT ST_TRANSFORM(
+                    ST_BUFFER(ST_TRANSFORM(ST_SETSRID(%s::geometry, 4326), %s), %s),
+                4326)""",
+                (geom.hex.decode("utf-8"), settings.ANALYSIS_COORDINATE_SYSTEM_SRID, width),
+            )
+            res = cursor.fetchone()
+            geom = GEOSGeometry(res[0], srid=4326)
     return geom
 
 
