@@ -197,6 +197,23 @@ class Command(BaseCommand):
             functions attached to the resource, as well as prevent some logging statements from printing to console.",
         )
 
+        parser.add_argument(
+            "-di",
+            "--defer_indexing",
+            action="store",
+            default=True,
+            dest="defer_indexing",
+            help="t/f - True(t) Defer indexing until all data is loaded (default).  Should speed up data load. False(f) to index resources and concepts incrementally during loading which allows users to search data while package load runs.",
+        )
+
+        parser.add_argument(
+            "-pi",
+            "--prevent_indexing",
+            action="store_true",
+            dest="prevent_indexing",
+            help="Prevents indexing the resources or concepts into Elasticsearch.  If set to True will override any 'defer_indexing' setting.",
+        )
+
         parser.add_argument(
             "-create_concepts",
             "--create_concepts",
@@ -226,6 +243,7 @@ class Command(BaseCommand):
     def handle(self, *args, **options):
         print("operation: " + options["operation"])
         package_name = settings.PACKAGE_NAME
+        celery_worker_running = task_management.check_if_celery_available()
 
         if options["operation"] == "setup":
             self.setup(package_name, es_install_location=options["dest_dir"])
@@ -248,7 +266,7 @@ class Command(BaseCommand):
             )
 
         if options["operation"] == "import_reference_data":
-            self.import_reference_data(options["source"], options["overwrite"], options["stage"], options["bulk_load"])
+            self.import_reference_data(options["source"], options["overwrite"], options["stage"], options["prevent_indexing"])
 
         if options["operation"] == "import_graphs":
             self.import_graphs(options["source"])
@@ -257,6 +275,19 @@ class Command(BaseCommand):
             self.export_graphs(options["dest_dir"], options["graphs"], options["type"])
 
         if options["operation"] == "import_business_data":
+            defer_indexing = True
+            if "defer_indexing" in options:
+                if isinstance(options["defer_indexing"], bool):
+                    defer_indexing = options["defer_indexing"]
+                elif str(options["defer_indexing"])[0].lower() == "f":
+                    defer_indexing = False
+
+            defer_indexing = defer_indexing and not options["bulk_load"] and not celery_worker_running
+            prevent_indexing = True if options["prevent_indexing"] else defer_indexing
+
+            if defer_indexing:
+                concept_count = models.Value.objects.count()
+                relation_count = models.ResourceXResource.objects.count()
 
             self.import_business_data(
                 options["source"],
@@ -266,8 +297,22 @@ class Command(BaseCommand):
                 options["create_concepts"],
                 use_multiprocessing=options["use_multiprocessing"],
                 force=options["yes"],
+                prevent_indexing=prevent_indexing,
             )
 
+            if defer_indexing and not prevent_indexing:
+                # index concepts if new concepts created
+                if concept_count != models.Value.objects.count():
+                    management.call_command("es", "index_concepts")
+                # index relations if new relations created via new r-i tiles created
+                if relation_count != models.ResourceXResource.objects.count():
+                    management.call_command("es", "index_resource_relations")
+                # index resources of this model only
+                path = utils.get_valid_path(options["config_file"])
+                mapping = json.load(open(path, "r"))
+                graphid = mapping["resource_model_id"]
+                management.call_command("es", "index_resources_by_type", resource_types=[graphid])
+
         if options["operation"] == "import_node_value_data":
             self.import_node_value_data(options["source"], options["overwrite"])
 
@@ -292,6 +337,7 @@ class Command(BaseCommand):
             self.create_mapping_file(options["dest_dir"], options["graphs"])
 
         if options["operation"] in ["load", "load_package"]:
+            defer_indexing = False if str(options["defer_indexing"])[0].lower() == "f" else True
             self.load_package(
                 options["source"],
                 options["setup_db"],
@@ -300,6 +346,7 @@ class Command(BaseCommand):
                 options["stage"],
                 options["yes"],
                 options["dev"],
+                defer_indexing,
             )
 
         if options["operation"] in ["create", "create_package"]:
@@ -315,7 +362,7 @@ class Command(BaseCommand):
         with open(os.path.join(dest_dir, "package_config.json"), "w") as config_file:
             try:
                 constraints = models.Resource2ResourceConstraint.objects.all()
-                configs = {"permitted_resource_relationships": constraints}
+                configs = {"permitted_resource_relationships": constraints, "business_data_load_order": []}
                 config_file.write(JSONSerializer().serialize(configs))
             except Exception as e:
                 print(e)
@@ -440,6 +487,7 @@ class Command(BaseCommand):
                 "extensions/datatypes",
                 "extensions/functions",
                 "extensions/widgets",
+                "extensions/etl_modules",
                 "extensions/css",
                 "extensions/bindings",
                 "extensions/card_components",
@@ -474,11 +522,24 @@ class Command(BaseCommand):
             self.export_package_settings(dest_dir, "true")
 
     def load_package(
-        self, source, setup_db=False, overwrite_concepts="ignore", bulk_load=False, stage_concepts="keep", yes=False, dev=False
+        self,
+        source,
+        setup_db=False,
+        overwrite_concepts="ignore",
+        bulk_load=False,
+        stage_concepts="keep",
+        yes=False,
+        dev=False,
+        defer_indexing=True,
     ):
 
         celery_worker_running = task_management.check_if_celery_available()
 
+        # only defer indexing if the celery worker ISN'T running because celery processes
+        # are async and we currently don't have a celery process to index data.  Once
+        # we do, then we can think about deferring indexing in the celery task as well.
+        defer_indexing = False if celery_worker_running else defer_indexing
+
         def load_ontologies(package_dir):
             ontologies = glob.glob(os.path.join(package_dir, "ontologies/*"))
             if len(ontologies) > 0:
@@ -573,7 +634,7 @@ class Command(BaseCommand):
             self.import_graphs(branches, overwrite_graphs=overwrite_graphs)
             self.import_graphs(resource_models, overwrite_graphs=overwrite_graphs)
 
-        def load_concepts(package_dir, overwrite, stage):
+        def load_concepts(package_dir, overwrite, stage, defer_indexing):
             file_types = ["*.xml", "*.rdf"]
 
             from time import time
@@ -588,7 +649,7 @@ class Command(BaseCommand):
             for path in concept_data:
                 if bar1 is None:
                     print(path)
-                self.import_reference_data(path, overwrite, stage, bulk_load)
+                self.import_reference_data(path, overwrite, stage, defer_indexing)
                 if bar1 is not None:
                     head, tail = os.path.split(path)
                     bar1.update(item_id=tail + (" " * 10))
@@ -601,7 +662,7 @@ class Command(BaseCommand):
             for path in collection_data:
                 if bar2 is None:
                     print(path)
-                self.import_reference_data(path, overwrite, stage, bulk_load)
+                self.import_reference_data(path, overwrite, stage, defer_indexing)
                 if bar2 is not None:
                     head, tail = os.path.split(path)
                     bar2.update(item_id=tail)
@@ -625,19 +686,27 @@ class Command(BaseCommand):
             load_mapbox_styles(basemap_styles, True)
             load_mapbox_styles(overlay_styles, False)
 
-        def load_business_data(package_dir):
+        def load_business_data(package_dir, prevent_indexing):
             config_paths = glob.glob(os.path.join(package_dir, "package_config.json"))
             configs = {}
             if len(config_paths) > 0:
                 configs = json.load(open(config_paths[0]))
 
             business_data = []
-            if "business_data_load_order" in configs and len(configs["business_data_load_order"]) > 0:
-                for f in configs["business_data_load_order"]:
-                    business_data.append(os.path.join(package_dir, "business_data", f))
+            if dev and os.path.isdir(os.path.join(package_dir, "business_data", "dev_data")):
+                if "business_data_load_order" in configs and len(configs["business_data_load_order"]) > 0:
+                    for f in configs["business_data_load_order"]:
+                        business_data.append(os.path.join(package_dir, "business_data", "dev_data", f))
+                else:
+                    for ext in ["*.json", "*.jsonl", "*.csv"]:
+                        business_data += glob.glob(os.path.join(package_dir, "business_data", "dev_data", ext))
             else:
-                for ext in ["*.json", "*.jsonl", "*.csv"]:
-                    business_data += glob.glob(os.path.join(package_dir, "business_data", ext))
+                if "business_data_load_order" in configs and len(configs["business_data_load_order"]) > 0:
+                    for f in configs["business_data_load_order"]:
+                        business_data.append(os.path.join(package_dir, "business_data", f))
+                else:
+                    for ext in ["*.json", "*.jsonl", "*.csv"]:
+                        business_data += glob.glob(os.path.join(package_dir, "business_data", ext))
 
             erring_csvs = [
                 path
@@ -668,13 +737,16 @@ class Command(BaseCommand):
                 ]
 
                 # assumes resources in csv do not depend on data being loaded prior from json in same dir
-                chord([import_business_data.s(data_source=path, overwrite=True, bulk_load=bulk_load) for path in valid_resource_paths])(
-                    package_load_complete.signature(kwargs={"valid_resource_paths": valid_resource_paths}).on_error(on_chord_error.s())
-                )
+                chord(
+                    [
+                        import_business_data.s(data_source=path, overwrite=True, bulk_load=bulk_load, prevent_indexing=False)
+                        for path in valid_resource_paths
+                    ]
+                )(package_load_complete.signature(kwargs={"valid_resource_paths": valid_resource_paths}).on_error(on_chord_error.s()))
             else:
                 for path in business_data:
                     if path not in erring_csvs:
-                        self.import_business_data(path, overwrite=True, bulk_load=bulk_load)
+                        self.import_business_data(path, overwrite=True, bulk_load=bulk_load, prevent_indexing=prevent_indexing)
 
             relations = glob.glob(os.path.join(package_dir, "business_data", "relations", "*.relations"))
             for relation in relations:
@@ -757,6 +829,9 @@ class Command(BaseCommand):
         def load_card_components(package_dir):
             load_extensions(package_dir, "card_components", "card_component")
 
+        def load_card_components(package_dir):
+            load_extensions(package_dir, "cards", "card_component")
+
         def load_search_components(package_dir):
             load_extensions(package_dir, "search", "search")
 
@@ -769,12 +844,15 @@ class Command(BaseCommand):
         def load_functions(package_dir):
             load_extensions(package_dir, "functions", "fn")
 
+        def load_etl_modules(package_dir):
+            load_extensions(package_dir, "etl_modules", "etl_module")
+
         def cache_graphs():
             management.call_command("cache", operation="graphs")
 
-        def update_resource_materialized_view():
+        def update_resource_geojson_geometries():
             with connection.cursor() as cursor:
-                cursor.execute("REFRESH MATERIALIZED VIEW mv_geojson_geoms;")
+                cursor.execute("SELECT * FROM refresh_geojson_geometries();")
 
         def load_apps(package_dir):
             package_apps = glob.glob(os.path.join(package_dir, "apps", "*"))
@@ -848,8 +926,10 @@ class Command(BaseCommand):
         load_functions(package_location)
         print("loading datatypes")
         load_datatypes(package_location)
+        print("loading etl modules")
+        load_etl_modules(package_location)
         print("loading concepts")
-        load_concepts(package_location, overwrite_concepts, stage_concepts)
+        load_concepts(package_location, overwrite_concepts, stage_concepts, defer_indexing)
         print("loading resource models and branches")
         load_graphs(package_location)
         print("loading resource to resource constraints")
@@ -859,7 +939,7 @@ class Command(BaseCommand):
         print("loading search indexes")
         load_indexes(package_location)
         print("loading business data - resource instances and relationships")
-        load_business_data(package_location)
+        load_business_data(package_location, defer_indexing)
         print("loading resource views")
         load_resource_views(package_location)
         print("loading apps")
@@ -875,9 +955,12 @@ class Command(BaseCommand):
             for css_file in css_files:
                 shutil.copy(css_file, css_dest)
         print("Refreshing the resource view")
-        update_resource_materialized_view()
+        update_resource_geojson_geometries()
         print("loading post sql")
         load_sql(package_location, "post_sql")
+        if defer_indexing is True:
+            print("indexing database")
+            management.call_command("es", "reindex_database")
         if celery_worker_running:
             print("Celery detected: Resource instances loading. Log in to arches to be notified on completion.")
         else:
@@ -925,7 +1008,9 @@ class Command(BaseCommand):
                 for graph in models.GraphModel.objects.filter(isresource=True).exclude(pk=settings.SYSTEM_SETTINGS_RESOURCE_MODEL_ID)
             ]
         if graphid is False and file_format != "json":
-            utils.print_message("Exporting data for all graphs is currently only supported for the json format")
+            utils.print_message(
+                "Exporting data for all graphs is currently only supported for the json format. Please specify a graphid with the -g flag."
+            )
             sys.exit()
         if graphid:
             graphids.append(graphid)
@@ -945,8 +1030,11 @@ class Command(BaseCommand):
                             ),
                             "w",
                         ) as f:
-                            file["outputfile"].seek(0)
-                            shutil.copyfileobj(file["outputfile"], f, 16 * 1024)
+                            if file_format == "tilexl":
+                                file["outputfile"].save(os.path.join(data_dest, file["name"]))
+                            else:
+                                file["outputfile"].seek(0)
+                                shutil.copyfileobj(file["outputfile"], f, 16 * 1024)
                 except KeyError:
                     utils.print_message("{0} is not a valid export file format.".format(file_format))
                     sys.exit()
@@ -959,16 +1047,24 @@ class Command(BaseCommand):
             )
             sys.exit()
 
-    def import_reference_data(self, data_source, overwrite="ignore", stage="stage", bulk_load=False):
+    def import_reference_data(self, data_source, overwrite="ignore", stage="stage", prevent_indexing=False):
         if overwrite == "":
             overwrite = "overwrite"
 
         skos = SKOSReader()
         rdf = skos.read_file(data_source)
-        ret = skos.save_concepts_from_skos(rdf, overwrite, stage)
+        ret = skos.save_concepts_from_skos(rdf, overwrite, stage, prevent_indexing)
 
     def import_business_data(
-        self, data_source, config_file=None, overwrite=None, bulk_load=False, create_concepts=False, use_multiprocessing=False, force=False
+        self,
+        data_source,
+        config_file=None,
+        overwrite=None,
+        bulk_load=False,
+        create_concepts=False,
+        use_multiprocessing=False,
+        force=False,
+        prevent_indexing=False,
     ):
         """
         Imports business data from all formats. A config file (mapping file) is required for .csv format.
@@ -978,17 +1074,17 @@ class Command(BaseCommand):
         if data_source.endswith(".jsonl"):
             print(
                 """
-WARNING: Support for loading JSONL files is still experimental. Be aware that
-the format of logging and console messages has not been updated."""
+                WARNING: Support for loading JSONL files is still experimental. Be aware that
+                the format of logging and console messages has not been updated."""
             )
             if use_multiprocessing is True:
                 print(
                     """
-WARNING: Support for multiprocessing files is still experimental. While using
-multiprocessing to import resources, you will not be able to use ctrl+c (etc.)
-to cancel the operation. You will need to manually kill all of the processes
-with or just close the terminal. Also, be aware that print statements
-will be very jumbled."""
+                    WARNING: Support for multiprocessing files is still experimental. While using
+                    multiprocessing to import resources, you will not be able to use ctrl+c (etc.)
+                    to cancel the operation. You will need to manually kill all of the processes
+                    with or just close the terminal. Also, be aware that print statements
+                    will be very jumbled."""
                 )
                 if not force:
                     confirm = input("continue? Y/n ")
@@ -1018,6 +1114,7 @@ will be very jumbled."""
             create_concepts = True
 
         if len(data_source) > 0:
+            transaction_id = uuid.uuid1()
             for source in data_source:
                 path = utils.get_valid_path(source)
                 if path is not None:
@@ -1028,6 +1125,8 @@ will be very jumbled."""
                         create_concepts=create_concepts,
                         create_collections=create_collections,
                         use_multiprocessing=use_multiprocessing,
+                        prevent_indexing=prevent_indexing,
+                        transaction_id=transaction_id,
                     )
                 else:
                     utils.print_message("No file found at indicated location: {0}".format(source))
